/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: 
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include <stdio.h>

#include "interf.h"
#include  "extern.h"

static XmStringCharSet	charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

extern Display *dpy;	       		/*Display*/
extern Widget  draw_area;
extern Widget  text_area;
extern int	screen;
extern GC draw_win_gc;                  /*the name says it all*/
extern GC text_win_gc;                  /*the name says it all*/
extern Widget app_shell;	        /*application shell widget*/



load_interp_from_file_motif(fname)
char 	*fname;
{
	int	components;
	int     comp_array[MAX_INTER];
	sprintf(Input_network_name, fname);
	read_graph_and_paths(fname, &No_inter, Inter_p,
                &components, comp_array, Graph);

	if((No_inter>MAX_INTER) || (components>MAX_COMP)){
         printf("Cannot handle %d intermediate goals\n", No_inter);
         printf("OR Cannot handle %d components.\n", components);
         exit(-1);
        }
	if(components>1) {
	 printf("More than 1 components, cannot handle this.\n");
	 exit(-1);
	}

        printf("Network read.\n");
	preprocess_motif();
}


load_workspace_from_file_motif(fname)
char*	fname;
{
	sprintf(Input_file_name, fname);
        read_obstacles_robot(fname, &ObjNum, ObjectwVertice, Objectw,
                             Begin_pos, End_pos,
                             &Num_cntl_points, Cntl_points,
                             &Link_length, &Link_vertices, Link,
                             &BaseX, &BaseY);
	strcpy(Workspace_name, Input_file_name);
	draw_workspace(dpy, XtWindow(draw_area), draw_win_gc, 
		screen, SCALE, SCALE, ObjNum, ObjectwVertice, Objectw);
}


        /*once a workspace and intermediate poinst have been read, 
	  build the tables, cspace, FK_inter etc*/ 
preprocess_motif()
{
	int 	i;

        printf("Constructing Tables and Cspace...\n");
	Read_Workspace = 1;
        preprocess(Workspace_name, Cspace_fname, Dist_fname, Pot_fname);
        for(i=0; i<No_inter; i++)
         FK(Inter_p[i], FK_inter[i]);
}



draw_workspace_motif()
{
	draw_workspace(dpy, XtWindow(draw_area), draw_win_gc,
                screen, SCALE, SCALE, ObjNum, ObjectwVertice, Objectw);
}


draw_robot_motif(conf)
configuration	conf;
{
	draw_robot(conf, dpy, XtWindow(draw_area), draw_win_gc,
		screen, SCALE, SCALE);
}


undraw_robot_motif(conf)
configuration	conf;
{
	undraw_robot(conf, dpy, XtWindow(draw_area), draw_win_gc,
                screen, SCALE, SCALE);
}


delay()
{
	int j, k;
	static int first=1;
	static int t;

	if(first) {t = atoi(getenv("DELAY")); first=0;}
	for(j=0; j<t; j++) for(k=0; k<t; k++);
}

connect_animate(path, pathl)
configuration 	*path;
int		pathl;
{
	int	i, j;
	char	s[10];

	draw_robot_motif(path[0]); 
	for(i=1; i<pathl; i++) {
	 undraw_robot_motif(path[i-1]); 
	 draw_robot_motif(path[i]);
	}
}

connect_animate_skip(path, pathl)
configuration   *path;
int             pathl;
{
        int     i, j;
        char    s[10];

        draw_robot_motif(path[0]);
        for(i=3; i<pathl; i+=3) {
         undraw_robot_motif(path[i-3]);
         draw_robot_motif(path[i]);
        }
	undraw_robot_motif(path[i-3]);
	draw_robot_motif(path[pathl-1]);
}




print_global_info(fp)
FILE *fp;
{
	int 	i, j, *tmp_int;
	double 	*tmp_double;
	int	type;

	
	fprintf(fp, "\n\nSTATISTICS\n");

	type = Gl_stats[0].type;
	i = 0;
	while(type != END_GLOBALS) {
	 fprintf(fp, "%-38s", Gl_stats[i].prompt);
	 switch (type) {
	  case INTEGER:	
			tmp_int = Gl_stats[i].addr;
	   		fprintf(fp, ": %d\n", *tmp_int);
			break;
	  case REAL:
			tmp_double = Gl_stats[i].addr;
			fprintf(fp, ": %lf\n", *tmp_double); 
                        break;
          case STRING:
			fprintf(fp, ": %s\n", Gl_stats[i].addr); 
                        break;
          case BOOLEAN:
			tmp_int = Gl_stats[i].addr;
			if(*tmp_int == 0)  
			 fprintf(fp, ":  FALSE\n");
			else 
			 fprintf(fp, ": TRUE\n");
			break;
	  case CONFIGURATION:
	 		tmp_int = Gl_stats[i].addr;
			fprintf(fp, ":");
			for(j=0; j<DOF; j++) 
			 fprintf(fp, " %d", tmp_int[j]); 
		 	fprintf(fp, "\n");
			break;
	  case NEW_LINE:
			fprintf(fp, "\n");
			break;
	 } /*switch*/
	 i++;
	 type = Gl_stats[i].type; 
	}	
}


text_window_global_info()
{
	int 	i, j, *tmp_int, height;
	double 	*tmp_double;
	int	type;
	char	text_string[150];
	
	XClearWindow(dpy, XtWindow(text_area));
        XFlush(dpy);

	height = 30;
        sprintf(text_string, "STATISTICS");
        XDrawString(dpy, XtWindow(text_area), text_win_gc, 10, 30,
                text_string, strlen(text_string));
	height += 20;
        XFlush(dpy);

	type = Gl_stats[0].type;
	i = 0;
	while(type != END_GLOBALS) {
	 switch (type) {
	  case INTEGER:	
			tmp_int = Gl_stats[i].addr;
			sprintf(text_string, "%-38s: %d",
				Gl_stats[i].prompt, *tmp_int);
			XDrawString(dpy, XtWindow(text_area), text_win_gc, 
			 10, height, text_string, strlen(text_string)); 
		        height += 20;
        		XFlush(dpy);
			break;
	  case REAL:
			tmp_double = Gl_stats[i].addr;
			sprintf(text_string, "%-38s: %lf",
				Gl_stats[i].prompt, *tmp_double);
			XDrawString(dpy, XtWindow(text_area), text_win_gc, 
			 10, height, text_string, strlen(text_string)); 
		        height += 20;
			XFlush(dpy);
                        break;
          case STRING:
			sprintf(text_string, "%-38s: %s",
				Gl_stats[i].prompt, Gl_stats[i].addr);
			XDrawString(dpy, XtWindow(text_area), text_win_gc,
                         10, height, text_string, strlen(text_string));
                        height += 20;
			XFlush(dpy);
                        break;
          case BOOLEAN:
			tmp_int = Gl_stats[i].addr;
			if(*tmp_int == 0)  
			 sprintf(text_string, "%-38s:   FALSE", 
				Gl_stats[i].prompt);
			else 
			 sprintf(text_string, "%-38s:   TRUE",
				Gl_stats[i].prompt);
			XDrawString(dpy, XtWindow(text_area), text_win_gc,
                         10, height, text_string, strlen(text_string));
                        height += 20;
                        XFlush(dpy);
			break;
	  case CONFIGURATION:
			/*
	 		tmp_int = Gl_stats[i].addr;
			fprintf(fp, ":");
			for(j=0; j<DOF; j++) 
			 fprintf(fp, " %d", tmp_int[j]); 
		 	fprintf(fp, "\n");
			*/
			break;
	  case NEW_LINE:
                        height += 20;
                        XFlush(dpy);
			break;
	 } /*switch*/
	 i++;
	 type = Gl_stats[i].type; 
	}	
}
			 	


/*****************************************************************************
	Draw the workspace on a given window with specified scale
	for the xaxis and the yaxis
*****************************************************************************/
void draw_workspace(display, win, gc, screen, xaxis_scale, yaxis_scale, 
				objnum, objectvertice, object)
Display* display;
Window win;
GC  gc;
int screen, xaxis_scale, yaxis_scale;
int objnum;
int objectvertice[MAXOBST];
IPoint object[MAXVERTEX][MAXOBST];
{
	XPoint dummyobject[MAXVERTEX];
	int i,j;

	XFlush(display);
	XSetForeground(display, gc, BlackPixel(display,screen));
  	for( j = 0; j < objnum; j++ ) {
   	 for( i = 0; i <= objectvertice[j]; i++) {
    	  dummyobject[i].x = xaxis_scale * object[i][j].x;
    	  dummyobject[i].y = yaxis_scale * object[i][j].y;
         }
    	XDrawLines(display, win, gc, &dummyobject[0], objectvertice[j]+1,
               CoordModeOrigin);
   	XFillPolygon(display, win, gc, &dummyobject[0], objectvertice[j] + 1,
                 Convex, CoordModeOrigin);
  	}
  	XFlush(display);
}

void draw_workspace_map(display, win, gc, screen, xaxis_scale, yaxis_scale, 
			wmap)
Display* display;
Window win;
GC  gc;
int screen, xaxis_scale, yaxis_scale;
int	wmap[DIM][DIM];
{
	int i,j;

	XFlush(display);
	XSetForeground(display, gc, BlackPixel(display,screen));
	for(i=0; i<DIM; i++) 
	 for(j=0; j<DIM; j++) 
	  if(wmap[i][j]) {
	  XDrawRectangle(display, win, gc, i*SCALE, j*SCALE, 
		xaxis_scale, yaxis_scale);
	  XFillRectangle(display, win, gc, i*SCALE, j*SCALE,
                xaxis_scale, yaxis_scale);
	}
  	XFlush(display);
}




