/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: 
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "defs.h"
#include "extern.h"

extern	void	qsort();
extern	int	network_connect();
extern	int 	search_network();

#define DEBUG_PL        	0
#define DEBUG_PLAN		0

#if  DEBUG_PL
        Display                 *display;
        int                     screen;
        Window                  win;
        GC                      gc;
        char                    s[10];
#endif

char				s[1000];
typedef struct info_cell {
        double  dist;
        int     num;
} info_cell;

/****************************************************************/

configuration 	tmp_path[MAXPATH];
int		tmp_path_len;
info_cell       D[MAX_INTER];

extern double find_length_poly_curve();
/****************************************************************/

plan_and_smooth()
{
	time_t	t_start, t_end, t1, t2;
	char 	text_string[250];
	int	i;
	double	len = 0.0;
		
        my_get_exact_time(t_start);
	/* find the path */
        Found = plan_with_network(Begin_pos, End_pos,
                        &Full_path_len, Full_path);
	if(!Found) return;

	/* copy the path for shortning and smoothing*/
	memcpy(Sm_path, Full_path, Full_path_len*sizeof(configuration));
	Sh_path_len = Sm_path_len = Full_path_len;

	my_get_exact_time(t1);
	smooth4(Sm_path, &Sm_path_len, 256);
	my_get_exact_time(t2);
	Timings[6] = (t2-t1)/TICKS_PER_SECOND;
        Lengths[6] = Sm_path_len;
	Lengths[7] = GRID(find_length_poly_curve(Full_path, Full_path_len));
	Lengths[8] = GRID(find_length_poly_curve(Sh_path, Sh_path_len));
	Lengths[9] = GRID(find_length_poly_curve(Sm_path, Sm_path_len));
	
/*
	print_global_info(stdout);
*/

	if(With_interface)
	 text_window_global_info();
	printf("Planning finished and smoothing finished.\n");
}


double find_length_poly_curve(path, path_len)
configuration   *path;
int             path_len;
{
        double  t, sum = 0;
        int     i, j, *pathi, *pathii;

        pathii = path[0];
        for (i=0; i<path_len-1; i++) {
         pathi = pathii;
         pathii = path[i+1];

         for(j=t=0; j<DOF; j++)
          t += SQUARE(pathi[j] - pathii[j]);
         sum += sqrt(t);
        }

        return sum;
}

int plan_with_network(begin_conf, end_conf, path_length, path)
configuration 	begin_conf, end_conf;
int		*path_length;
configuration	path[MAXPATH];
{
	int 	flag1, flag2, flag3, i, j;
	int 	conf1_num, conf2_num, third_path_len;
	time_t 	t1, t2;

#if DEBUG_PL
        display = XOpenDisplay(NULL);
        screen = DefaultScreen(display);
        gc = XCreateGC(display, RootWindow(display,screen), 0, NULL);
        win = XCreateSimpleWindow(display, RootWindow(display, screen),
                           0,0,640,640,DefaultDepth(display,screen),
                           BlackPixel(display, screen),
                           WhitePixel(display, screen));
        XSelectInput(display, win, ExposureMask | ButtonPressMask);
        XMapWindow(display,win);
        XFlush(display);
        printf("\tPress <return> for graphics.");
        gets(s);
#endif
	         /*exact connection established with the begining conf*/
	*path_length = 0;
	ON_COMMENTS(printf("Trying to connect beginning conf.\n");)
	my_get_exact_time(t1);
	flag1 = heur_stratergy(begin_conf, &conf1_num, 1,
	    			&tmp_path_len, tmp_path);
	ON_COMMENTS(printf("Heuristic connect, flag1= %d and path %d\n", 
			flag1, tmp_path_len);) 
	my_get_exact_time(t2);
        Timings[1] = (t2-t1)/TICKS_PER_SECOND;
	if(!flag1) {*path_length = 0; return 0;}
	Lengths[1] = tmp_path_len;
	for(i=tmp_path_len-1, j=0; i>=0 ; i--, j++) 
	 assign_conf(tmp_path[i], path[j]);
	*path_length = tmp_path_len;

		/*control point connection established with the end*/
	ON_COMMENTS(printf("Trying to connect final conf.\n");)
	my_get_exact_time(t1);
	flag3 = heur_stratergy(end_conf, &conf2_num, 1, 
				&tmp_path_len, tmp_path);
	ON_COMMENTS(printf("Heuristic connect, flag3= %d and path %d \n", 
				flag3, tmp_path_len);) 
	my_get_exact_time(t2);
        Timings[3] = (t2-t1)/TICKS_PER_SECOND;
	if(!flag3) {*path_length = 0; return 0;} 
	Lengths[3] = tmp_path_len;
	
		/*plan the path in the network*/
	ON_COMMENTS(printf("Trying to connect in the network %d the %d.\n",
				conf1_num, conf2_num);)
	my_get_exact_time(t1);
	flag2 = network_connect(conf1_num, conf2_num, 
				&third_path_len, &(path[*path_length][0]));
	ON_COMMENTS(printf("Heuristic connect, flag2= %d and path %d \n", 
				flag2, third_path_len);) 
	my_get_exact_time(t2);
        Timings[2] = (t2-t1)/TICKS_PER_SECOND;
	Timings[0] = Timings[1] + Timings[2] + Timings[3];
	if(!flag2) {*path_length = 0; return 0;}
	Lengths[2] = third_path_len;
	*path_length = (*path_length) + third_path_len;

		/*concatenate the paths*/
	ON_COMMENTS(printf("Concatenating the paths %d %d.\n",
					tmp_path_len, *path_length);)
	concat(tmp_path, path, tmp_path_len, path_length);
	ON_COMMENTS(printf("Concatenation of paths finished (%d).\n",
				*path_length);)
	Lengths[0] = *path_length;

			/*if connection has been established 
			  our data structures must be updated, ADAPTIVE*/
	return 1;	
}



/*******************************************************************/


static  char    vst[MAX_INTER];
static  int     back[MAX_INTER];

	/*input: n1, n2 (nodes in the network)
	  output: path, pathl  (path in graph and length)
	  returns 1 iff n1 and n2  are connected
	*/
int search_network(n1, n2, pathl, path)
int	n1, n2, *pathl, *path;
{
	int	i, j, k, pl;
	static	int	list1[MAX_INTER], list2[MAX_INTER];
	int	*l1=list1, *l2=list2, *tmpl, nl1, nl2;
	cellp	*c1;

	if(n1==n2) { 
	 *pathl = 1; path[0] = n1; return 1;
	}
	/* initialize all nodes to not-visited */
	for(i=0; i<No_inter; i++) {vst[i] = 0; back[i] = -1;}
	/* initialize the list to {n1} */
	l1[0] = n1;
	nl1 = 1;	
	vst[n1] = 1;
	/* while the node n2 has not been reached,
	   go one step further */
new_list:
	nl2 = 0;
	for(i=0; i<nl1; i++)
	 for(c1=Graph[l1[i]]; c1; c1=c1->next)
	  if(!vst[j=c1->final]) {
	   l2[nl2++] = j;
	   vst[j] = 1;
	   back[j] = l1[i]; /* node from which j was reached */
	   if(j == n2) /* found */
	    goto found;
	  }
	  /* new list l2 is ready here */
	  if(!nl2) /* whole component has been examined */
	   return 0;
	  /* replace old list */
	  tmpl = l1; l1 = l2; l2 = tmpl;
	  nl1 = nl2;
	  goto new_list;
found:
	for(pl=1, path[0]=n2; (j=back[path[pl-1]])!=n1;
		path[pl++] = j);
	path[pl++] = n1;
	for(i=0; i<(k=pl-1-i); i++) {
	 j = path[i]; path[i] = path[k]; path[k] = j;
	}
	*pathl = pl;
	/* return success */
	return 1;
}


	/*produce the actual paths among nodes conf1_num and conf2_num
	  using the precomputed network.
	  The path and its length are returned*/
int network_connect(conf1_num, conf2_num, path_len, path)
int 		conf1_num, conf2_num;
int 		*path_len;
configuration 	path[MAXPATH];
{
	int 		link_num, i, tmp_len, found;
	static int	links[MAX_INTER];

	if(conf1_num==conf2_num) {
	 *path_len = 1;
	 assign_conf(Inter_p[conf1_num], path[0]);
	 return 1;
	}
		/*search for the node path*/
	found = search_network(conf1_num, conf2_num, &link_num, links);
	if(!found) return 0;

		/*a path has been found, get the path together*/
	*path_len = 0;
	for(i=0; i<link_num-1; i++) {
	 if(get_path(Graph, links[i], links[i+1], 
			&tmp_len, &(path[*path_len][0]))) { 
	  *path_len = *path_len + tmp_len;
	 } 
	 else return 0; 
	}
	return 1;
}


/*******************************************************************/

int cell_compare(x, y)
info_cell 	*x, *y;
{
	if(x->dist>y->dist) return 1;
	else if(x->dist == y->dist) return 0;
	else return -1;
}



	/*"exact" specifies if an exact or a control point
	  connection is required*/
int heur_stratergy(final_conf, con_conf_num, exact, path_len, path)
configuration 	final_conf;
int 	      	*con_conf_num, exact, *path_len;
configuration 	path[MAXPATH];
{
	int 	i, j, j_all, flag, conf_found, length, k, l, m, tmp_length,b ;
	int	same_conf;
	double 	d;
	FPoint	fk_conf[NBP], ws_pos[NBP];
	static 	configuration ttmp_path[MAXPATH];
	configuration conf, tconf;
	time_t  t1, t2;

	print_conf(final_conf);
	FK(final_conf, fk_conf);

	flag = 0;
		/*the first step is the easy planner*/
 	my_get_exact_time(t1);
        t2 = t1;
 	for(j=j_all=0; j<No_inter; j++)  {
         distance_ws(fk_conf, FK_inter[j], d);
         if(d<Connect_distance) {
          D[j_all].num = j;
          D[j_all++].dist = d;
         }
        }
        qsort(D, j_all, sizeof(info_cell), cell_compare);

	for(j=0;(j<j_all) && (!flag); j++)
         flag = connect_simple(Inter_p[D[j].num],final_conf,1,path_len,path);
        if(flag) conf_found = D[j-1].num;

		/*if not found, do a random walk and try again*/  

	for(k=0; (t2-t1 < Query_time) && (!flag); k++) {
	 assign_conf(final_conf, ttmp_path[0]);
		/*execute a predefined number of random walks steps*/
	 length = mrandom(Random_connect_length)+10;
	 random_bounce(final_conf, length, &tmp_length, &(ttmp_path[1][0]));
	 tmp_length++;
	  
/*
	  ON_COMMENTS(printf("Random walk length %d, steps %d (l=%d)\n", 
			length, tmp_length, l);)
*/

	 if(tmp_length> 1) { 

	 FK(ttmp_path[tmp_length-1], fk_conf);
	 for(j=j_all=0; j<No_inter; j++)  {
          distance_ws(fk_conf, FK_inter[j], d);
          if(d<Connect_distance) {
           D[j_all].num = j;
           D[j_all++].dist = d;
          }
         }
         qsort(D, j_all, sizeof(info_cell), cell_compare);

	 for(j=0; (j<j_all) && (!flag); j++) 
         flag = connect_simple(Inter_p[D[j].num],
	        	ttmp_path[tmp_length-1], 1,path_len,path);
         if(flag) {
	  conf_found = D[j-1].num;
	  for(l=tmp_length-1, j=0; l>=0; l--, j++) 
	   assign_conf(ttmp_path[l], path[*path_len+j]); 
	  *path_len += tmp_length; 
	 }
	 } /*if suc walk*/
          my_get_exact_time(t2);
	}
	
	if(flag) {*con_conf_num = conf_found; return 1;}
	return 0;
}
	
