/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/



#include "interf.h"
#include "extern.h"


static XmStringCharSet	charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

Display *dpy;			/*Display*/
int	screen;
GC draw_win_gc;                 /*the name says it all*/
GC text_win_gc;                 /*the name says it all*/
Widget main_window;		/*MainWindow*/
Widget main_window2;		/*MainWindow*/
Widget draw_area;		/*DrawingArea widget where the text is*/
Widget text_area;
Widget app_shell;		/*ApplicationShell*/
Widget app_shell2;		/*ApplicationShell*/
extern Widget create_application();/*create top level window*/ 
extern Widget create_text_area();/*create top level window*/ 

Widget quit_dialog;		/*dialogs*/
Widget load_workspace_dialog;
Widget load_interp_dialog;
Widget pos_dialog;
int    Pos_Flag;
Widget easy_plan_dialog;
Widget show_path_dialog;
Widget animate_dialog;


main_motif(argc,argv) 
unsigned int argc;
char **argv;
{
    int n;
    Arg args[MAX_ARGS];         /*arg list*/
    XtAppContext appCon;
    XWindowAttributes window_attr;
    
    /*	
     * Initialize toolkit and open the display.
     */
    XtToolkitInitialize();
    appCon = XtCreateApplicationContext();

    if ((dpy = XtOpenDisplay(appCon, NULL, "NewRandPlanner", "NewRandPlanner", 
			NULL, 0, &argc, argv)) == NULL)
    {
    	XtWarning ("example: Can't open display, exiting...");
    	exit(0);
    }
    screen = DefaultScreen(dpy);

    /*
     * Create application shell.
     */
    app_shell = XtAppCreateShell("NewRandPlanner", "NewRandPlanner", 
				applicationShellWidgetClass, dpy, NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNmappedWhenManaged, False); n++;
    XtSetValues(app_shell, args, n);

    /*
     * Create and realize main application window.
     */
    main_window = create_application(app_shell);
    XtRealizeWidget(app_shell);
    XResizeWindow(dpy, XtWindow(app_shell), 660, 700);

    init_misc_draw();    	/* do post XtRealizeWidget stuff */




    /*create and realize the main text window*/
    app_shell2 = XtAppCreateShell("Textwindow", "Textwindow",
			applicationShellWidgetClass, dpy, NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNmappedWhenManaged, False); n++;
    XtSetValues(app_shell2, args, n);

    main_window2 = create_text_area(app_shell2);
    XtRealizeWidget(app_shell2);
    XResizeWindow(dpy, XtWindow(app_shell2), 580, 800);

    init_misc_text();    	/* do post XtRealizeWidget stuff */
    
    /*	
     * Get and dispatch events.
     */
    XtMapWidget(app_shell);
    XtMapWidget(app_shell2);
    load_workspace_from_file_motif(Workspace_name);
    XtAppMainLoop(appCon);


}




/*
 * init_misc - this function initializes some stuff that needs to
 *	       be done AFTER the widgets have been realized.
 */
init_misc_draw()
{
    Pixmap button_pixmap;
    Pixel fg, bg;
    int n;
    Arg args[MAX_ARGS];
    XGCValues gcvalues;

    XColor scrn_def_ret, exact_def_ret;

    if (!XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
                          "blue", &scrn_def_ret, &exact_def_ret))
    {
        printf("Unable to allocate blue.\n");
	exit(1);
    }

    gcvalues.foreground = scrn_def_ret.pixel;
    gcvalues.line_width = 2;

    draw_win_gc = XCreateGC(dpy, XtWindow(draw_area), 
			    GCForeground | GCLineWidth, &gcvalues);

	/*create a copy for rubber banding*/
    /*
     * Set the name of the Application for the Window Manager
     */
    XStoreName(dpy, XtWindow(app_shell), "Probabilistic Path Planner");
}


init_misc_text()
{
    Pixmap button_pixmap;
    Pixel fg, bg;
    int n;
    Arg args[MAX_ARGS];
    XGCValues gcvalues;

    XColor scrn_def_ret, exact_def_ret;

    if (!XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
                         "blue", &scrn_def_ret, &exact_def_ret))
    {
        printf("Unable to allocate blue.\n");
        exit(1);
    }

    gcvalues.foreground = scrn_def_ret.pixel;
    gcvalues.line_width = 2;

    text_win_gc = XCreateGC(dpy, XtWindow(text_area),
                            GCForeground | GCLineWidth, &gcvalues);
    /*
     * Set the name of the Application for the Window Manager
     */
    XStoreName(dpy, XtWindow(app_shell2), "Messages");
}





/*
 * menu_selection - callback for any menu selection
 */
menu_selection(w, client_data, call_data) 
Widget w;
caddr_t client_data;       /*This could be OPEN, NEW, QUIT, etc*/
caddr_t call_data;
{
    Arg args[MAX_ARGS];
    int n;
    XmString mesg_string = NULL;

    switch ((int) client_data)
    {
	case LOAD_WORKSPACE:
	    printf("LOAD_WORKSPACE.\n");
            XtManageChild(load_workspace_dialog);
	    break;
	case LOAD_GRAPH:
	    printf("LOAD_GRAPH.\n");
            XtManageChild(load_interp_dialog);
	    break;
	case PRINT:
	    printf("PRINT.\n");
	    break;
	case QUIT:
	    printf("QUIT.\n");
	    XtManageChild(quit_dialog);
	    break;
	case COM_CSPACE:
	    printf("COM_CSPACE.\n");
	    preprocess_motif();
	    break;
	case INIT_POS:
	    printf("INIT_POS.\n");
	    Pos_Flag = 0; 	/*init pos considered, for the button*/
	    XtManageChild(pos_dialog);
	    break;
	case FINAL_POS:
	    printf("FINAL_POS.\n");
	    Pos_Flag = 1; 	/*final pos considered, for the button*/
	    XtManageChild(pos_dialog);
	    break;
	case EASY_PLAN:
	    printf("EASY_PLAN.\n");
	    plan_and_smooth();
	    break;
	case WRITE_PATH:
	    printf("WRITE_PATH.\n");
	    write_path("path", Sh_path_len, Sh_path , DOF);
	    break; 
	case ANIMATE_F_PATH:
	    printf("ANIMATE_F_PATH \n");
	    connect_animate(Full_path, Full_path_len);
	    break;
	case ANIMATE_SH_PATH:
	    printf("ANIMATE_SH_PATH \n");
	    printf("Skipping configurations..\n");
	    connect_animate_skip(Full_path, Full_path_len);
	    break;
	case ANIMATE_SM_PATH:
	    printf("ANIMATE_SM_PATH \n");
	    connect_animate(Sm_path, Sm_path_len);
	    break;
	default:
	    printf("ERROR, in menu_selection, unknown item.\n");
	    break;
    }
}



/********************draw window call back functions****************/
draw_win_input(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;      /*data from widget class*/
{
    XmDrawnButtonCallbackStruct *cb_struct;
    XEvent *the_event;

    cb_struct = (XmDrawnButtonCallbackStruct *) call_data;
    the_event = (XEvent *) cb_struct->event;

    switch (the_event->type)
    {
         case ButtonPress:
             button_press(the_event);
             break;
         case ButtonRelease:
	     printf("Button Release.\n");
             break;
         case KeyPress:
             printf("KeyPress event.\n");
             break;
         default:
             printf("Unknown event type in draw_win_input.\n");
             break;
    }
}


/*
 * button_press - this function handles a left button down event in the 
 *                drawing window.
 */
button_press(our_event)
XEvent *our_event;
{
    XButtonPressedEvent *the_event;

    the_event = (XButtonPressedEvent *) our_event;
    if (the_event->button != Button1)  /*we don't listen to nothin' but But 1*/
	return;

    printf("Button Press event at x=%d, y=%d.\n", the_event->x, the_event->y);
}


draw_win_expose(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;      /*data from widget class*/
{
    char *mesg;
    XWindowAttributes attr_ret;
    XmDrawnButtonCallbackStruct *cb_struct;
    XExposeEvent *the_event;

    /*
     * first check if this is the last expose event in the series, and then
     * repaint the window 
     */
    cb_struct = (XmDrawnButtonCallbackStruct *) call_data;
    the_event = (XExposeEvent *) cb_struct->event;

    if (the_event->count == 0)
    {
	printf("Draw window expose event.\n");
	draw_workspace_motif();
        XFlush(dpy);
    }
}


draw_win_resize(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;      /*data from widget class*/
{
    printf("Draw window resize event.\n");
}




/**************************msg window******************/

text_win_input(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;     
{
    XmDrawnButtonCallbackStruct *cb_struct;
    XEvent *our_event;
    XButtonPressedEvent *the_event;

    cb_struct = (XmDrawnButtonCallbackStruct *) call_data;
    our_event = (XEvent *) cb_struct->event;
    the_event = (XButtonPressedEvent *) our_event;


    switch (our_event->type)
    {
         case ButtonPress:
	     if(the_event->button == Button1) {
    		char        str[50];

    		printf("Msg window input event.\n");
		text_window_global_info();	
	      }
	      break;
	  default:
	      break;
	}
	
}

text_win_expose(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;      /*data from widget class*/
{
    XmDrawnButtonCallbackStruct *cb_struct;
    XExposeEvent *the_event;

    /*
     * first check if this is the last expose event in the series, 
	and then repaint the window
     */
    cb_struct = (XmDrawnButtonCallbackStruct *) call_data;
    the_event = (XExposeEvent *) cb_struct->event;

    if (the_event->count == 0)
    {
        printf("Text window expose event.\n");
	text_window_global_info();
        XFlush(dpy);
    }
}



text_win_resize(w, client_data, call_data)
Widget w;               /*widget id*/
caddr_t client_data;    /*data from application*/
caddr_t call_data;      /*data from widget class*/
{
    printf("Msg window Resize event.\n");
}

