/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Brian Wilson
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/

#include "interf.h"

static XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

extern Display *dpy;	       /*Display*/
extern GC draw_win_gc;                 /*the name says it all*/
extern GC text_win_gc;                 /*the name says it all*/
extern Widget main_window;             /*MainWindow*/
extern Widget main_window2;             /*Main Text Window*/
extern Widget draw_area;	       /*DrawingArea widget where the text is*/
extern Widget text_area;
extern Widget app_shell;	       /*application shell widget*/
extern Widget app_shell2;	       /*application shell widget*/
Widget create_application();           /*create top level window*/ 
Widget create_text_area();           /*create top level window*/ 
Widget create_menus(); 		       /*create menus, add callbacks*/

extern int draw_win_expose(); 		/* some callback funcions */
extern int draw_win_resize(); 
extern int draw_win_input(); 
extern int text_win_expose(); 		/* some callback funcions */
extern int text_win_resize(); 
extern int text_win_input(); 
extern int menu_selection(); 
extern int operation_sel(); 

extern Widget quit_dialog;             /*dialogs*/
extern Widget load_workspace_dialog;
extern Widget load_interp_dialog;
extern Widget com_cspace_dialog;
extern Widget pos_dialog;
extern Widget show_path_dialog;
extern Widget animate_dialog;

					/* dialog creation functions */
extern Widget create_quit_dialog();             
extern Widget create_load_workspace_dialog();
extern Widget create_load_interp_dialog();
extern Widget create_pos_dialog();
extern Widget create_show_path_dialog();
extern Widget create_animate_dialog();


extern int Pos_Flag;



/*
 * create_application - this function creates the application window's children
 */
Widget create_application(parent) 
Widget parent;			/*parent widget*/
{
    Widget main_window;		/*MainWindow*/
    Widget menu_bar;		/*MenuBar*/
    Widget frame;		/*Frame*/
    Widget main_form;		/*form*/
    Widget draw_area_frame;	/*text area Frame*/
    Widget tool_form;		/*form with all the tools*/
    Widget tool_form_draw_area;
    Widget button;		/*PushButtonGadget*/
    Arg	args[MAX_ARGS];		/*arg list*/
    int n;			/*arg count*/
    
    /*	
     * Create MainWindow.
     */
    n = 0;
    main_window = XmCreateMainWindow(parent, "main_win", args, n);
    XtManageChild(main_window);

    menu_bar = create_menus(main_window);    /* create menus, add callbacks */
    /*
     * Create Frame MainWindow for the outdented look
     */
    n = 0;
    XtSetArg(args[n], XmNshadowThickness, 2);  n++;
    XtSetArg(args[n], XmNshadowType, XmSHADOW_OUT);  n++;
    frame = XmCreateFrame(main_window, "frame", args, n);
    XtManageChild(frame);

    n = 0;
    main_form = XmCreateForm(frame, "main_form", args, n);
    XtManageChild(main_form);

    /*
     * This frame gives the drawing area the indented look 
     */
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 5); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 5); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 5); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, 5); n++;
    XtSetArg(args[n], XmNshadowThickness, 2);  n++;
    XtSetArg(args[n], XmNshadowType, XmSHADOW_IN);  n++;
    draw_area_frame = XmCreateFrame(main_form, "draw_area_frame", args, n);
    XtManageChild(draw_area_frame);

    /*
     * Create the main drawing area
     */
    n = 0;
    draw_area = XmCreateDrawingArea(draw_area_frame, "draw_area", args, n);
    XtManageChild(draw_area);
    XtAddCallback(draw_area, XmNexposeCallback, draw_win_expose, NULL);
    XtAddCallback(draw_area, XmNresizeCallback, draw_win_resize, NULL);
    XtAddCallback(draw_area, XmNinputCallback, draw_win_input, NULL);


    /*	
     * Set MainWindow areas
     */
    XmMainWindowSetAreas(main_window, menu_bar, NULL, NULL, NULL, frame);

    /*
     * create all the dialogs
    */
    quit_dialog = create_quit_dialog();
    load_workspace_dialog = create_load_workspace_dialog();
    load_interp_dialog = create_load_interp_dialog();
    pos_dialog = create_pos_dialog();
    show_path_dialog = create_show_path_dialog();
    animate_dialog = create_animate_dialog();

    /*
     * Return MainWindow.
     */
    return(main_window);
}

    
/*
 * create_menus - this function creates all the pull down menus and attaches
 *                the callbacks to them.  It returns the menu bar widget.
 */
Widget create_menus(main_window)
Widget main_window;
{
    Widget menu_bar;	        /*MenuBar*/
    Widget menu_pane;		/*MenuPane*/
    Widget cascade;		/*CascadeButton*/
    Widget button;
    Widget toggle;
    Widget separator;
    Widget edit_field;
    Arg	args[MAX_ARGS];
    XmString but_str;
    int n;

    /*
     * Create MenuBar in MainWindow.
     */
    n = 0;
    menu_bar = XmCreateMenuBar(main_window, "menu_bar", args, n); 
    XtManageChild(menu_bar);
    
    /*	
     * Create "Environment" PulldownMenu.
     */
    n = 0;
    menu_pane = XmCreatePulldownMenu(menu_bar, "menu_pane", args, n);
    
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    cascade = XmCreateCascadeButton (menu_bar, "Environment", args, n);
    XtManageChild(cascade);

/*
    n = 0;
    but_str = XmStringCreateLtoR("Load Workspace", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    button = XmCreatePushButton (menu_pane, "Load Workspace", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, LOAD_WORKSPACE);
*/

    n = 0;
    but_str = XmStringCreateLtoR("Load Graph", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    button = XmCreatePushButton (menu_pane, "Load Graph", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, LOAD_GRAPH);

/*
    n = 0;
    button = XmCreatePushButton (menu_pane, "Print", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, PRINT);
*/

    n = 0;
    separator = XmCreateSeparatorGadget(menu_pane, "separator", args, n);
    XtManageChild(separator);

    n = 0;
    button = XmCreatePushButton (menu_pane, "Quit", args, n);
    XtManageChild (button);
    XtAddCallback (button, XmNactivateCallback, menu_selection, QUIT);

    
    /*	
     * Create "Select pos" PulldownMenu.
     */
    n = 0;
    menu_pane = XmCreatePulldownMenu(menu_bar, "menu_pane", args, n);
    
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    cascade = XmCreateCascadeButton (menu_bar, "Preprocess", args, n);
    XtManageChild(cascade);

/*
    n = 0;
    but_str = XmStringCreateLtoR("Compute Cspace", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Compute Cspace", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, COM_CSPACE);
*/

    n = 0;
    but_str = XmStringCreateLtoR("Initial Position", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Initial Position", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, INIT_POS);

    n = 0;
    but_str = XmStringCreateLtoR("Final Position", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Final Position", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, FINAL_POS);
	
    /*	
     * Create "Plan" button.
     */
    n = 0;
    menu_pane = XmCreatePulldownMenu(menu_bar, "menu_pane", args, n);

    n = 0;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    cascade = XmCreateCascadeButton (menu_bar, "Plan", args, n);
    XtManageChild(cascade);

    n = 0;
    but_str = XmStringCreateLtoR("Plan", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Easy Plan", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, EASY_PLAN);

    /*
     * Create "Show" button.
     */
    n = 0;
    menu_pane = XmCreatePulldownMenu(menu_bar, "menu_pane", args, n);

    n = 0;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    cascade = XmCreateCascadeButton (menu_bar, "Animate", args, n);
    XtManageChild(cascade);

    n = 0;
    but_str = XmStringCreateLtoR("Full Path", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Full Path", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, 
			ANIMATE_F_PATH);


    n = 0;
    but_str = XmStringCreateLtoR("Shortened Path", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Shortened", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, 
			ANIMATE_SH_PATH);

    n = 0;
    but_str = XmStringCreateLtoR("Smoothed path", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Smoothed path", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection, 
                        ANIMATE_SM_PATH);
	
    n = 0;
    but_str = XmStringCreateLtoR("Write path", charset);
    XtSetArg(args[n], XmNlabelString, but_str); n++;
    XtSetArg(args[n], XmNsubMenuId, menu_pane);  n++;
    button = XmCreatePushButton (menu_pane, "Write path", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, menu_selection,
                        WRITE_PATH);


    n = 0;
    XtSetArg (args[n], XmNmenuHelpWidget, cascade);  n++;
    XtSetValues (menu_bar, args, n);
  
    return(menu_bar);
}


/*
 * create_text_window
 */
Widget create_text_area(parent) 
Widget parent;			/*parent widget*/
{
    Widget main_window;		/*MainWindow*/
    Widget frame;		/*Frame*/
    Widget main_form;		/*form*/
    Widget text_area_frame;	/*text area Frame*/
    Widget button;		/*PushButtonGadget*/
    Arg	args[MAX_ARGS];		/*arg list*/
    int n;			/*arg count*/
    
    /*	
     * Create MainWindow.
     */
    n = 0;
    main_window = XmCreateMainWindow(parent, "main_win", args, n);
    XtManageChild(main_window);

    /*
     * Create Frame MainWindow for the outdented look
     */
    n = 0;
    XtSetArg(args[n], XmNshadowThickness, 2);  n++;
    XtSetArg(args[n], XmNshadowType, XmSHADOW_OUT);  n++;
    frame = XmCreateFrame(main_window, "frame", args, n);
    XtManageChild(frame);

    n = 0;
    main_form = XmCreateForm(frame, "main_form", args, n);
    XtManageChild(main_form);

    /*
     * create a drawing window for text 
     */
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 5); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 5); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 5); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, 5); n++;
    XtSetArg(args[n], XmNshadowThickness, 2);  n++;
    XtSetArg(args[n], XmNshadowType, XmSHADOW_IN);  n++;
    text_area_frame = XmCreateFrame(main_form, "text_area_frame", args, n);
    XtManageChild(text_area_frame);

    /*
     * Create the main text area
     */
    n = 0;
    text_area = XmCreateDrawingArea(text_area_frame, "text_area", args, n);
    XtManageChild(text_area);
    XtAddCallback(text_area, XmNexposeCallback, text_win_expose, NULL);
    XtAddCallback(text_area, XmNresizeCallback, text_win_resize, NULL);
    XtAddCallback(text_area, XmNinputCallback, text_win_input, NULL);

    /*
     * Return MainWindow.
     */
    return(main_window);
}

    

/*************************************************************************
	Creating various dialogs
*************************************************************************/



/************************************************************************
	 this function sets the indicated dialog's name to
	 the indicated string 
************************************************************************/
set_dialog_wm_name(the_dialog, the_name)
Widget the_dialog;
char *the_name;
{
    	int n;
    	Arg args[MAX_ARGS];
	
    	n = 0;
    	XtSetArg(args[n], XmNtitle, the_name); n++;
    	XtSetValues(XtParent(the_dialog), args, n);
}


/*************************************************************************
	load the workspace 
*************************************************************************/
handle_load_workspace_dialog(the_dialog, client_data, call_data)
Widget the_dialog;
caddr_t client_data;
caddr_t call_data;
{
    	XmFileSelectionBoxCallbackStruct *fcb;    /*data from the call back*/
    	char *fname;

    	switch ((int) client_data) {
        case CANCEL:
	    printf("...must be explicitly unmanaged.\n");
            XtUnmanageChild(the_dialog);
	    break;
        case OK:
            fcb = (XmFileSelectionBoxCallbackStruct *) call_data;
            XmStringGetLtoR(fcb->value, charset, &fname);
	    printf("File selected is: %s.\n", fname);
            XtUnmanageChild(the_dialog);
	    load_workspace_from_file_motif(fname);
            break;
        default:
            printf("unknown load_workspace_dialog button was pushed.");
            break;
    	}
}

Widget create_load_workspace_dialog()
{
    	int n;
    	Widget the_dialog;
    	XmString mesg_string = NULL;
    	XmString mask_string = NULL;
    	Arg args[MAX_ARGS];

    	mesg_string = XmStringCreateLtoR("Select A Workspace To Read In:", 
				charset);
    	mask_string = XmStringCreateLtoR("*.env", charset);
    	n = 0;
    	XtSetArg(args[n], XmNselectionLabelString, mesg_string); n++;
    	XtSetArg(args[n], XmNdirMask, mask_string); n++;
    	the_dialog = XmCreateFileSelectionDialog(app_shell, 
				"Load Worspace",args,n);
        set_dialog_wm_name(the_dialog, "Load Workspace");
    	XtAddCallback(the_dialog, XmNokCallback, 
				handle_load_workspace_dialog, OK);
    	XtAddCallback(the_dialog, XmNcancelCallback, 
				handle_load_workspace_dialog, CANCEL);
    	if(mesg_string)
	 XtFree(mesg_string);
        if (mask_string)
	 XtFree(mask_string);
    	return(the_dialog);
}


/*************************************************************************
	load the intermediate points 
*************************************************************************/
handle_load_interp_dialog(the_dialog, client_data, call_data)
Widget the_dialog;
caddr_t client_data;
caddr_t call_data;
{
    	XmFileSelectionBoxCallbackStruct *fcb;    /*data from the call back*/
    	char *fname;

    	switch ((int) client_data) {
        case CANCEL:
	    printf("...must be explicitly unmanaged.\n");
            XtUnmanageChild(the_dialog);
	    break;
        case OK:
            fcb = (XmFileSelectionBoxCallbackStruct *) call_data;
            XmStringGetLtoR(fcb->value, charset, &fname);
	    printf("File selected is: %s.\n", fname);
            XtUnmanageChild(the_dialog);
	    load_interp_from_file_motif(fname);
            break;
        default:
            printf("unknown load_workspace_dialog button was pushed.");
            break;
    	}
}


Widget create_load_interp_dialog()
{
    	int n;
    	Widget the_dialog;
    	XmString mesg_string = NULL;
    	XmString mask_string = NULL;
    	Arg args[MAX_ARGS];

    	mesg_string = XmStringCreateLtoR("Select Graph To Be Read In:", 
				charset);
    	mask_string = XmStringCreateLtoR("*", charset);
    	n = 0;
    	XtSetArg(args[n], XmNselectionLabelString, mesg_string); n++;
    	XtSetArg(args[n], XmNdirMask, mask_string); n++;
    	the_dialog = XmCreateFileSelectionDialog(app_shell, 
				"Load Graph",args,n);
        set_dialog_wm_name(the_dialog, "Load Workspace");
        XtAddCallback(the_dialog, XmNokCallback, 
				handle_load_interp_dialog, OK);
    	XtAddCallback(the_dialog, XmNcancelCallback, 
				handle_load_interp_dialog, CANCEL);
    	if (mesg_string)
	 XtFree(mesg_string);
    	if (mask_string)
	 XtFree(mask_string);
    	return(the_dialog);
}



/*************************************************************************
	handle the quit dialog	
*************************************************************************/

handle_quit_dialog(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
    	int result;

    	switch ((int) client_data) {
        case NO:
            break;
        case YES:
            exit(0);
            break;
        case CANCEL:
            return;
            break;
        default:
            printf("unknown quit_dialog button pushed.");
            break;
    	}
}

Widget create_quit_dialog() 
{
    	Widget quit_dialog;
    	Widget button;
    	Widget edit_field;
    	Widget quit_label_text;
    	Arg	args[MAX_ARGS];
    	int n;
    	XmString mesg_string = NULL;
    	XmString yes_mesg_string = NULL;
    	XmString no_mesg_string = NULL;
    	XmString cancel_mesg_string = NULL;
    	XmString button_string = NULL;

    	mesg_string = XmStringCreateLtoR("Sure you want to Quit?", charset);
    	yes_mesg_string = XmStringCreateLtoR("Yes", charset);
    	no_mesg_string = XmStringCreateLtoR("No", charset);
    	cancel_mesg_string = XmStringCreateLtoR("Cancel", charset);

    	n=0;
    	XtSetArg(args[n], XmNmessageString, mesg_string); n++;
    	XtSetArg(args[n], XmNokLabelString, yes_mesg_string); n++;
    	XtSetArg(args[n], XmNcancelLabelString, cancel_mesg_string); n++;
    	XtSetArg(args[n], XmNhelpLabelString, no_mesg_string); n++;
    	quit_dialog = XmCreateQuestionDialog(app_shell, 
			"quit_dialog", args, n);
    	set_dialog_wm_name(quit_dialog, "Quit");
    	XtAddCallback(quit_dialog, XmNokCallback, handle_quit_dialog, YES);
    	XtAddCallback(quit_dialog, XmNcancelCallback, 
			handle_quit_dialog, CANCEL);
        XtAddCallback(quit_dialog, XmNhelpCallback, handle_quit_dialog, NO);

        if (mesg_string)
         XtFree(mesg_string);
        if (yes_mesg_string)
         XtFree(yes_mesg_string);
    	if (no_mesg_string)
         XtFree(no_mesg_string);
    	if (cancel_mesg_string)
         XtFree(cancel_mesg_string);
    	if (button_string)
         XtFree(button_string);

    	return(quit_dialog);
}


/**********************************************************************
	To manage the "init_pos" dialog some global Widgets are 
	used. Same for "final pos"
**********************************************************************/
Widget  		scale[DOF], pos_dialog;	
Widget			msg_configuration1, msg_configuration2;	
configuration		tmp_conf;
static char descr_dof[DOF][50] = {
        		 "angle 1st link",
        		 "angle 2nd link",
        		 "angle 3rd link",
        		 "angle 4th link",
        		 "angle 5th link",
        		 "angle 6th link",
        		 "angle 7th link"
        		};



scaleCB(w, client_data, call_data)
Widget 			w;               
int     		client_data;
XmScaleCallbackStruct   *call_data;
{
	FPoint	ws_conf[NBP];

   	printf("Dof #%d: new value %d.\n", client_data, call_data->value);
	undraw_robot_motif(tmp_conf);
	draw_workspace_motif();
	tmp_conf[client_data] = call_data->value;
	draw_robot_motif(tmp_conf);
	printf(" 	New conf: ");
	print_conf(tmp_conf);
	FK(tmp_conf, ws_conf);	
	printf("	Collision: out %d, obstacles %d, auto %d \n", 
		out_ws(ws_conf), VC(ws_conf, tmp_conf),
		auto_collision(tmp_conf, ws_conf)); 
}


button_1CB(w, client_data, call_data)
Widget                  w;
int			client_data;
XmScaleCallbackStruct   *call_data;
{
      	int 		i, *pos;
	
	if(Pos_Flag==0)  /*initial position treated*/
	 pos = Begin_pos;
	else 		    /*final position treated*/
	 pos = End_pos;

	undraw_robot_motif(tmp_conf);
	print_conf(pos);
	assign_conf(pos, tmp_conf);
	draw_robot_motif(pos);
	for(i=0; i<DOF; i++) 
	 XmScaleSetValue(scale[i], pos[i]);
	printf("Scales set to their initial values.\n");
}

button_2CB(w, client_data, call_data)
Widget                  w;
int			client_data;
XmScaleCallbackStruct   *call_data;
{
        int 		i;
	int		*pos;

	if(Pos_Flag==0)  /*initial position treated*/
         pos = Begin_pos;
        else                /*final position treated*/
         pos = End_pos;

	undraw_robot_motif(tmp_conf);
        for(i=0; i<DOF; i++) 
	 XmScaleGetValue(scale[i], &(pos[i]));
	printf("Configuration selected is: ");
	print_conf(pos);

		/*explicitely close the dialog*/
	XtUnmanageChild(pos_dialog);
}

button_3CB(w, client_data, call_data)
Widget                  w;
int			client_data;
XmScaleCallbackStruct   *call_data;
{
	undraw_robot_motif(tmp_conf);
        printf("No change in the position.");
                /*explicitely close the dialog*/
        XtUnmanageChild(pos_dialog);
}


Widget create_pos_dialog()
{
	Widget	rowcol, sep[DOF];
	Widget  label1, button_1, button_2, button_3;
	char	str_s[50], str_b[200];
	Arg	args[MAX_ARGS];
	int	n, i;

	n = 0;
	XtSetArg(args[n], XmNheight, 90*DOF+60); n++;
	XtSetArg(args[n], XmNwidth, 340); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_MODELESS); n++;
	pos_dialog = XmCreateBulletinBoardDialog(app_shell, 
		"pos_dialog", args, n);
	set_dialog_wm_name(pos_dialog, "Configuration Adjustment");

	n = 0;
	rowcol = XmCreateRowColumn(pos_dialog, "rowcol", args, n);
	XtManageChild(rowcol);
	

	for(i=DOF-1; i>=0; i--) {
	 n = 0; 
         XtSetArg(args[n], XmNtitleString, 
	  		XmStringCreate(descr_dof[i], charset)); n++;
	 XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	 XtSetArg(args[n], XmNshowValue, True); n++;
	 XtSetArg(args[n], XmNmaximum, DIM - 1); n++;
	 XtSetArg(args[n], XmNscaleMultiple, 1); n++;
	 XtSetArg(args[n], XmNscaleWidth, 300); n++;
	 scale[i] = XmCreateScale(rowcol, "scale", args, n); 
	 XtManageChild(scale[i]);
	 XtAddCallback(scale[i], XmNvalueChangedCallback, scaleCB, i);
	 XmScaleSetValue(scale[i], 0);
	 if(i> 0) {
	 n = 0;
	 sep[i] = XmCreateSeparatorGadget(rowcol, "separator", args, n);
	 XtManageChild(sep[i]);
	 }
	}
			/*display the configuration and its validity*/
	sprintf(str_b, "Configuration: ");
        n = 0;
        XtSetArg(args[n], XmNlabelString,
                XmStringCreateLtoR(str_b, charset)); n++;
        msg_configuration1 = XmCreateLabel(rowcol, "label1", args, n);
        XtManageChild(msg_configuration1);
	sprintf(str_b, "Collision:          Self-collision: ");
        n = 0;
        XtSetArg(args[n], XmNlabelString,
                XmStringCreateLtoR(str_b, charset)); n++;
        msg_configuration2 = XmCreateLabel(rowcol, "label1", args, n);
        XtManageChild(msg_configuration2);

			/*create buttons*/
			/*load existing configuration*/
	n = 0;
	XtSetArg(args[n], XmNtitleString,
                        XmStringCreate("Get Conf", charset)); n++;
	button_1 = XmCreatePushButton(rowcol, 
			"Get current configuration", args, n);
	XtManageChild(button_1);
	XtAddCallback(button_1, XmNactivateCallback, button_1CB, 
			Pos_Flag);

			/*accept a configuration*/
	n = 0;
        XtSetArg(args[n], XmNtitleString,
                        XmStringCreate("Done ", charset)); n++;
	button_2 = XmCreatePushButton(rowcol, 
			"Accept configuration",args,n);
        XtManageChild(button_2);
        XtAddCallback(button_2, XmNactivateCallback, button_2CB, 
				Pos_Flag);

			/*cancel the transaction*/
	n = 0;
        XtSetArg(args[n], XmNtitleString,
                        XmStringCreate("Cancel ", charset)); n++;
        button_2 = XmCreatePushButton(rowcol, "Cancel", args, n);
        XtManageChild(button_2);
        XtAddCallback(button_2, XmNactivateCallback, button_3CB,
                                Pos_Flag);

	return(pos_dialog);
}


Widget create_show_path_dialog() {};
Widget create_animate_dialog() {};
Widget create_redraw_w_dialog() {};
Widget create_redraw_r_dialog() {};

/*****************************************************************************/
