/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#define ON_PAPER(x)  {if (DEBUG_PAPER) {x};}

#define ONCE_ONLY       {static int done=0;if(!done) { done=1;
#define ONCE_END        }}


#define SAFETY 		1
#define TRUNC(x) 	( ( (x) & mask1 ) ? 0 : ( (x) & mask2) ? 127 : (x) )
#define FRACTIONAL(x) 	(((x)>=0)?((x)-(int)(x)):((x)-(int)(x)+1))
#define my_MIN(A,B)     ((A)>(B)?(B):(A))
#define my_MAX(A,B)     ((A)>(B)?(A):(B))
#define SIGN(a)       	((a) < 0 ? -1 : (a) > 0 ? 1 : 0)
#define ROUND(a)	((int) ((a) +0.5))
/*
#define GRID(x)       	(int) ( (x) + 0.00001 )
*/
#define GRID(x)         ((int) ((x) +0.5))
#define my_ABS(a)        	((a) < 0 ? -(a) : (a))
#define SQUARE(a)     	((a)*(a))
#define DET(a,b,c,d)    ((a)*(d)-(b)*(c))
#define FZERO(x)        ((((x)>-SMALL)&&((x)<SMALL))?1:0)
	/* x in the interval (a,b) */
#define IN(x,a,b) ((((x)<=(b))&&((x)>=(a))))
	/* x in the interval (a,b) or (b,a) */
#define IIN(x,a,b) (IN((x),(a),(b)) || IN((x),(b),(a)))
	/* the intervals (a,b) and (c,d) overlap (or touch) */
#define OVERLAP(a,b,c,d)\
        (IIN((a),(c),(d))||IIN((b),(c),(d))||\
         IIN((c),(a),(b))||IIN((d),(a),(b)))


/*************************************************************************/

#define get(type, n)    ((type*)malloc((n)*sizeof(type)))

		/*assign a configuration to another*/
#define assign_conf(a, b)       bcopy((a), (b), sizeof(configuration))

/************************************************************************/


#define set_2d_bitmap(name, i, j, val)\
{	int j_j;\
	if(val) {\
	 j_j = (j) >> 5;\
	 (name)[(i)][j_j] |= ( 1<<( (j) -((j_j)<<5)) );\
	}\
}\

#define read_2d_bitmap(name, i, j)\
	((name)[(i)][((j) >> 5)] & (1<< ((j)-(((j) >> 5)<<5) ) ))

/*************************************************************************/

#define set_3d_bitmap(name, i, j, k, val)\
{	int k_k; \
	if(val) {\
	 k_k = (k) >> 5;\
	 (name)[i][j][k_k] |= (1<<( (k) - (k_k<<5) ) );\
	}\
}\

#define read_3d_bitmap(name, i, j, k)\
	((name)[i][j][(k) >> 5] & (1<< ((k)-(((k) >> 5)<<5) ) ))

/*************************************************************************/
                /*print a configuration*/

#define print_conf(a)\
{	int i_i; \
        for (i_i=0; i_i<DOF; i_i++)\
         printf("%d ", a[i_i]);\
        printf("\n");\
}\

		/*print a workspace position*/

#define print_ws_pos(a)\
{	int i_i; \
	for (i_i=0; i_i<NBP; i_i++)\
         printf("(%.2lf,%.2lf) ", a[i_i].x, a[i_i].y);\
	printf("\n");\
}\


/*************************************************************************/
		/*make sure an angle is in [0, DIM-1]*/	

#define adjust_angle(angle)		(((angle)+100*DIM) % DIM)

		/*convert to int a workspace position*/
#define double2int_ws_pos(double_pos, int_pos)\
{	int i_i; \
	for(i_i=0; i_i<NBP; i_i++){\
	 int_pos[i_i].x = ROUND(double_pos[i_i].x);\
	 int_pos[i_i].y = ROUND(double_pos[i_i].y);\
	}\
}\

/*************************************************************************/
		/*compute a heuristic distance*/

#define distance_ws(w_pos1, w_pos2, distance)\
{	double a_a, b_b; \
	int	i_i;\
	(distance) = 0.0; \
	for(i_i=0; i_i<NBP; i_i++){\
	 a_a = (w_pos1)[i_i].x-(w_pos2)[i_i].x; \
	 b_b = (w_pos1)[i_i].y-(w_pos2)[i_i].y; \
	 (distance) += a_a*a_a + b_b*b_b; \
	} \
}\

/*************************************************************************/

#define theta2DIM(theta) \
	(((theta)>0)?\
		ROUND((theta)*DIM/TWO_PI):\
		ROUND((TWO_PI+theta)*DIM/TWO_PI))

/************************************************************************
	define macros used when doing autocollisions
*************************************************************************/


#define    collide_seg(ws_pos_a, ws_pos_b, ws_pos_c, ws_pos_d, bool) \
	{ double a, b, c, d, z, w, det, dets, dett, s, t;\
	a = ((ws_pos_a).x - (ws_pos_b).x); \
	b = ((ws_pos_d).x - (ws_pos_c).x); \
	c = ((ws_pos_a).y - (ws_pos_b).y); \
	d = ((ws_pos_d).y - (ws_pos_c).y); \
	z = ((ws_pos_d).x - (ws_pos_b).x); \
	w = ((ws_pos_d).y - (ws_pos_b).y); \
	det = DET(a,b,c,d); \
	if(!FZERO(det)) {\
	 dets = DET(z,b,w,d);\
	 dett = DET(a,z,c,w);\
	 s = dets/det; \
	 t = dett/det; \
	 if(IN(s,0,1) && IN(t,0,1)) bool = 1;\
	 else bool=0;\
	} \
	else {\
	 if(!FZERO(DET(a,c,(ws_pos_c).x-(ws_pos_a).x,\
		(ws_pos_c).y-(ws_pos_a).y)))\
	  bool = 0;\
	 else {\
	  bool = OVERLAP((ws_pos_a).x,(ws_pos_b).x,(ws_pos_c).x,(ws_pos_d).x)\
	   && OVERLAP((ws_pos_a).y,(ws_pos_b).y,(ws_pos_c).y,(ws_pos_d).y);\
	 }\
	}\
	}

#define check_in_cone(angle, cone, check_angle, bool)\
	{ int x=(check_angle)-(angle);\
	  x = adjust_angle(x);\
	  bool = IN(x,0,(cone)) || IN(x,DIM-(cone),DIM-1);\
	}\
\


#define	CLOSE_DISTANCE	4
#define	point_near_segment(pt, a, b, bool)\
	{ double x, y, dxa, dxb, dab;\
	  x = pt.x - a.x;\
	  y = pt.y - a.y;\
	  dxa = sqrt(x*x+y*y);\
	  x = pt.x - b.x;\
	  y = pt.y - b.y;\
	  dxb = sqrt(x*x+y*y);\
	  x = b.x - a.x;\
	  y = b.y - a.y;\
	  dab = sqrt(x*x+y*y);\
	  bool = (dxa + dxb - dab <= CLOSE_DISTANCE);\
	}


#define cmp_confs(a,b, bool) \
	{int i; \
	 bool = 1; \
	 for(i=0; (bool) && (i<DOF); i++) \
	  if((a)[i] != (b)[i]) (bool)=0; \
	}
