/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include "defs.h"
#include "globals.c"

#define DEBUG_PAPER 		1
#define DEBUG_PL                0

extern 	int 	dpoint_compare_new();

#if  DEBUG_PL
        Display                 *display;
        int                     screen;
        Window                  win;
        GC                      gc;
        XGCValues 		gcvalues;
        char                    s[10];
#endif


typedef struct dpoint {
        double  dist;
        int     ind;
} dpoint;



/**************Numerical Paramaters*************************************/
	/*number of easy connections tried per node during the 
	  network construction phase*/
int		Heur_num = 20;  

	/*when expanding dof is allowed a range its value +/- Deviation*/
int             Deviation = 10;	

	/*components will less than Cut_off*(total elements) disgarded
	  BUT when Cut_off = -1.0 all the components but the largest 
	  are discarded*/
double		Cut_off = -1.0;	

	/*confs with distance>Connect_distance not tried for connection*/
/* global variable
int		Connect_distance = BIG_DIST; 
*/
	/*initial seed for random number generator*/
int		Seed = 1994;

	/*Number of configurations added*/
int		M;
/**********************************************************************/


int             Num_comp[MAX_COMP_INITIAL];
dpoint		D[MAX_INTER];
int             In_component_array[MAX_INTER];
int             Components, New_components;
cellp           *Graph[MAX_INTER], *New_graph[MAX_INTER];
int		New_no_inter;
configuration	New_inter_p[MAX_INTER];
configuration 	aux_path[MAXPATH];
int		Shrink = 0;

			/*for expansion heuristic*/
int		Con_per_node[MAX_INTER];
int		Hist[MAX_INTER];

static char 	cspace_fname[100] = "cspace.dat", 
		dist_fname[100] = "dist.dat";

/**********************************************************************/

int	compar_int(x, y)
int	*x, *y;
{
	if(*x<*y) return 1;
	if(*x>*y) return -1;
	return 0;
}

/**********************************************************************/
void read_args(argc, argv, do_preprocess, cspace_fname, dist_fname, 
	seed, heur_num, cut_off, deviation, 
	connect_distance)	
int argc;
char *argv[];
char *cspace_fname, *dist_fname;
int  *do_preprocess, *seed, *heur_num, *deviation;
int  *connect_distance;
double	*cut_off;
{
	int i;
        for(i=0; i<argc; i++) {
	  if(strcmp(argv[i], "-P")==0)
           *do_preprocess = atoi(argv[i+1]);
	  if(strcmp(argv[i], "-CF")==0)
	   cspace_fname = strcpy(cspace_fname, argv[i+1]);
	  if(strcmp(argv[i], "-DF")==0)
	   dist_fname = strcpy(dist_fname, argv[i+1]);
	  if(strcmp(argv[i], "-H")==0)
	   *heur_num= atoi(argv[i+1]);
	  if(strcmp(argv[i], "-DEV")==0)
           *deviation = atoi(argv[i+1]);
	  if(strcmp(argv[i], "-S")==0)
	   *seed = atoi(argv[i+1]);
	  if(strcmp(argv[i], "-CUT")==0)
	   *cut_off= atof(argv[i+1]);
          if(strcmp(argv[i], "-DD")==0)
	   *connect_distance = atoi(argv[i+1]);
	}
}


void main(argc,argv)
int argc;
char *argv[];
{
	int     	new_inter, new,
			Num_comp_srt[MAX_COMP_INITIAL];
	int		count, max_count, step, sum;
	time_t		t1, t2, tt1, tt2;
	int 		i, j, c, max_comp, limit;
	cellp		*tmp;
	FILE 		*fp, *fpp;
	configuration	son;
	FPoint		ws_son[NBP];


	if(argc<4) {
 	 printf("USE: s_connect <env><out_graph><stat_file><N><M> \n");

	 printf("Optional parameters: \n");
	 printf("\t -P  <1 to use  preprocessing> (default = 0)\n");
	 printf("\t -CF <cspace file>		(default = cspace.dat)\n");
	 printf("\t -DF <distance file>		(default = dist.dat)\n");
	 printf("\t -S  <seed>			(default = 1994)\n");	
	 printf("\t -H  <heur num for con> 	(default = 20) \n");	
	 printf("\t -CUT <cut_off>  		(default = -1.0) \n");	
	 printf("\t -DEV <devaition in exp>	(default = 20) \n");	
	 printf("\t -DD <distance away>		(default = 200000)\n"); 
	 exit(-1);
	}

	read_args(argc, argv, &Use_preprocess, cspace_fname, dist_fname, 
			&Seed, &Heur_num,
			&Cut_off, &Deviation, 
			&Connect_distance);

		/*read workspace and initialize*/
	Read_Workspace = 1;
	preprocess(argv[1], cspace_fname, dist_fname);

	srandom(Seed);
	srand48(Seed+1);

	No_inter = atoi(argv[4]);
	M = atoi(argv[5]);
	if((No_inter +M)>MAX_INTER) {
         printf("Can not handle that many nodes.\n");
         exit(-1);
        }
		/*create the configurations*/
	my_get_exact_time(t1);
	step = 0;
        while(step<No_inter) {
         random_conf(son);

	 
	 /* print_conf(son); */
         FK(son, ws_son);
	 /* print_ws_pos(ws_son); */
         if(!(out_ws(ws_son) ||
            VC(ws_son, son) ||
            auto_collision(son, ws_son))) {
          assign_conf(son, Inter_p[step]);
          step++;
         }
        }
        my_get_exact_time(t2);
/*
	write_path("init_nodes", No_inter, Inter_p, DOF);
*/

		/*open statistics file*/
	fp = fopen(argv[3], "a");
        if(!fp) {
         printf("Statistics file %s could not be opened.\n", argv[3]);
         exit(-1);
        }
	fprintf(fp, "\nStatistics from the run.\n");
        fprintf(fp, "-------------------------\n");
	fprintf(fp, "Parameters\n");
	fprintf(fp, "Name of the workspace file %s.\n", argv[1]);
        fprintf(fp, "Connections attempted per conf %d\n", Heur_num);
	fprintf(fp, "Connect_Distance %d\n", Connect_distance);
        fprintf(fp, "Deviation is %d.\n", Deviation);
	fprintf(fp, "Shrinking of expansion phase %d\n", Shrink);
	fprintf(fp, "Seed %d\n", Seed);
	if(Cut_off != -1.0) 
	 fprintf(fp, "Cut_off %d.\n", GRID(No_inter*Cut_off));
	else 
	 fprintf(fp, "Cut_off: all but the largest.\n");
	fprintf(fp, "Creation:  time there %f sec\n",
                                (t2-t1)/TICKS_PER_SECOND);


#if     DEBUG_PL
        display = XOpenDisplay(NULL);
        screen = DefaultScreen(display);
        gcvalues.line_width = 2;
        gcvalues.foreground = BlackPixel(display, screen);
        gc = XCreateGC(display, RootWindow(display,screen), 0, &gcvalues);
        win = XCreateSimpleWindow(display, RootWindow(display, screen),
                            0,0,640,640,DefaultDepth(display,screen),
                            BlackPixel(display, screen),
                            WhitePixel(display, screen));
        XSelectInput(display, win, ExposureMask | ButtonPressMask);
        XMapWindow(display,win);
        XFlush(display);
#endif

	
	/**************************************************************/
	fprintf(fp, "Connect phase\n");
	printf("Connect phase\n");
	my_get_exact_time(t1);

	init_store_str(&(Graph[0]), No_inter);
        for(i=0; i<No_inter; i++)
         FK(Inter_p[i], FK_inter[i]);

	try_to_connect_before(0, No_inter, Heur_num, fp);
	Components = 
	     connected_components(Graph, No_inter, In_component_array); 
	my_get_exact_time(t2);
	printf("Components have been found.\n");

	fprintf(fp, "Run:  time spent in connect phase %f sec\n", 
				(t2-t1)/TICKS_PER_SECOND);
	for(i=c=0; i<No_inter; i++) if(!Graph[i]) c++;
	fprintf(fp, "%d out of %d rows had no connections\n", c, No_inter);
	fprintf(fp, "Number of connected Components %d\n", Components);
	for(i=0; i<Components; i++)  Num_comp[i] = 0;
                /*find the number of elements in each component*/
        for(i=0; i<No_inter; i++) Num_comp[In_component_array[i]]++;
	
        max_comp = -1;
        for(i=0; i<Components; i++) Num_comp_srt[i] = Num_comp[i];
        qsort(Num_comp_srt, Components, sizeof(int), compar_int);
        for(i=0; i<Components; i++)
         if(Num_comp_srt[i]>max_comp) max_comp = Num_comp_srt[i];
        for(i=0; i<Components; i++) {
         fprintf(fp, "%5d ",  Num_comp_srt[i]);
	 if ((i % 11 == 0) && (i!=0)) fprintf(fp, "\n");
	}

/*
	for(i=0; i<Components; i++) 
	 if(Num_comp[i]>1) 
	  fprintf(fp, "Component %d has %d nodes \n", i, Num_comp[i]);
*/

	/**************************************************************/

	   	/*get num of connections per node */
	max_count = 0;
	for(i=0; i<No_inter; i++) Hist[i] = 0; 
        for(i=0; i<No_inter; i++) {
         for(tmp = Graph[i], count = 0; tmp; tmp = tmp->next) count++;
	 if (count>max_count) max_count = count;
         Con_per_node[i] = count;
	 Hist[count]++;
        }

        fprintf(fp, "Augment phase\n");
        my_get_exact_time(t1);
        new_inter = add_confs_rw(Con_per_node, fp);
        my_get_exact_time(t2);
        fprintf(fp, "Augmentation done in %f sec \n",
                               (t2-t1)/TICKS_PER_SECOND);
        fprintf(fp,"%d new configurations were added\n",new_inter-No_inter);

/*
	write_path("added_nodes", new_inter - No_inter, 
		&(Inter_p[No_inter][0]), DOF);
*/

	/****************************************************************/
	fprintf(fp, "Reconnect Phase\n");
	my_get_exact_time(t1);
	if(Shrink)
	 try_to_connect_after(No_inter, new_inter, Heur_num, 
			In_component_array, fp);
	else 
	 try_to_connect_before(No_inter, new_inter, Heur_num, fp);

	new = new_inter - No_inter;
	No_inter = new_inter;
	Components = 
	     connected_components(Graph, No_inter, In_component_array); 
	my_get_exact_time(tt2);
	fprintf(fp, "Expand new node connection took %f secs.\n",
                                (tt2-t1)/TICKS_PER_SECOND);

	printf("Components have been found\n");
	fprintf(fp, "%d new points were added for a total of %d.\n", 
				new, No_inter);
	for(i=c=0; i<No_inter; i++) if(!Graph[i]) c++;
        fprintf(fp, "%d out of %d rows had no connections\n", c, No_inter);
	fprintf(fp, "Number of connected Components %d\n", Components);
        for(i=0; i<Components; i++)  Num_comp[i] = 0;
        for(i=0; i<No_inter; i++) Num_comp[In_component_array[i]]++;


	max_comp = -1;
	for(i=0; i<Components; i++) Num_comp_srt[i] = Num_comp[i];
	qsort(Num_comp_srt, Components, sizeof(int), compar_int); 
        for(i=0; i<Components; i++)
         if(Num_comp_srt[i]>1) {
/*
          fprintf(fp, "Component %d has %d nodes \n", i, Num_comp[i]);
*/
	  if(Num_comp_srt[i]>max_comp) max_comp = Num_comp_srt[i];
	 }
	fprintf(fp, "<comp> ");
	for(i=0; i<Components; i++) {
         fprintf(fp, "%5d ",  Num_comp_srt[i]);
	 if ((i % 11 == 0) && (i!=0)) fprintf(fp, "\n");
        }

	/***************************************************************/

	if(Cut_off != -1.0)
         fprintf(fp, "Cut_off %d.\n", GRID(No_inter*Cut_off+1));
        else
         fprintf(fp, "Cut_off: all but the largest.\n");

		/*keep in a file the components that are larger than 
		  Cut_off*No_inter */ 
	if(Cut_off != -1.0) 
	 limit = GRID(Cut_off*No_inter+1);
	else limit = max_comp - 2;
	my_get_exact_time(t1);
  	printf("Covert limit %d\n", limit);
        convert_cutoff(No_inter, &New_no_inter,
                          Inter_p, New_inter_p,
                          Components, &New_components,
                          In_component_array,
                          Graph, New_graph, limit);
	my_get_exact_time(t2);

		/*print some data for error checking purposes*/
	fprintf(fp, "FOR ERROR CHECKING (after isolation time taken %f)\n",
					(t2-t1)/TICKS_PER_SECOND); 
	fprintf(fp, "Number of configurations %d\n", New_no_inter);
	fprintf(fp, "Number of connected Components %d\n", New_components);
	for(i=0; i<New_components; i++)  Num_comp[i] = 0;
                /*find the number of elements in each component*/
        for(i=0; i<New_no_inter; i++) Num_comp[In_component_array[i]]++;
        for(i=0; i<New_components; i++) 
         if(Num_comp[i] >1) 
	  fprintf(fp, "Component %d has %d nodes \n", i, Num_comp[i]);

        fflush(fp);
        fclose(fp);
	write_graph_and_paths(argv[2], New_no_inter, New_inter_p, 
		New_components, In_component_array, New_graph);

}

int dpoint_compare_new(x, y)
dpoint *x, *y;
{
        if (x->ind > y->ind) return -1;
        else if (x->ind == y->ind) return 0;
        else return 1;
}


int dpoint_compare(x, y)
dpoint *x, *y;
{
        if (x->dist > y->dist) return 1;
        else if (x->dist == y->dist) return 0;
        else return -1;
}


try_to_connect_before(low_i, high_i, heur_num, fp)
int	low_i, high_i, heur_num;
FILE	*fp;
{
	int 	i, j, path_len = 0, j_all, jcount;
	double 	d;
        int     suc_connections = 0, tried_connections = 0;
        int     total_sorted = 0;

	printf("Connect before\n");
	if(low_i==high_i) return 1;

	for(i=low_i; i<high_i; i++) { 
		/*find the confs relatively close to i and sort, look
		  only at the lower triangle*/
	 for(j=j_all=0; j<i; j++) { 
	  distance_ws(FK_inter[i], FK_inter[j], d);
	  if (d<Connect_distance) {
	   D[j_all].dist = d;
	   D[j_all++].ind = j;
	  }
	 }
	 qsort(D, j_all, sizeof(dpoint), dpoint_compare);
	 ON_PAPER(total_sorted += j_all;);
	
	 jcount = my_MIN(heur_num, j_all);
	 for(j=0; j<jcount; j++) {
	  ON_PAPER(tried_connections++;);
          if(connect_simple(Inter_p[i], Inter_p[D[j].ind],
                                1, &path_len, aux_path)) {
           set_store_str(Graph, i, D[j].ind, 1, CONNECT_LINE, 0, NULL);
	   ON_PAPER(suc_connections++;);
          }
         } /*for j*/

	} /* for i*/

	ON_PAPER(
        fprintf(fp, "BEFORE:Total number of configurations sorted:  %d\n", 
			total_sorted);
        fprintf(fp, "BEFORE:Total number of configurations tried:   %d\n", 
			tried_connections);
        fprintf(fp, "BEFORE:Total number of configurations succeded:%d\n", 
			suc_connections);
        fprintf(fp, "BEFORE:Total connections %d, con/node = %f\n",
                	suc_connections, 
			(suc_connections*1.0)/(high_i-low_i));
	)

	printf("Try to connect (before) finished.\n");
}



try_to_connect_after(low_i, high_i, heur_num, in_component_array, fp)
int     low_i, high_i, heur_num;
int     in_component_array[MAX_INTER];
FILE	*fp;
{
        int     i, j, path_len = 0, j_all, cur_comp, jcount, tried;
	int	n_labels_i, labels_i[MAX_COMP_INITIAL], k, fl, c_j;
        double  d;
	int	suc_connections = 0, tried_connections = 0;
	int	total_sorted = 0;

	printf("In trying to connect after\n");
	if(low_i == high_i) return;
		/*initialization has been done by expand small components*/

       	for(i=low_i; i<high_i; i++) {
                /*find the confs relative close to i and sort*/
	 cur_comp = in_component_array[i];
         for(j=j_all=0; j<i; j++) {
		/*try only if connection is beneficial ie with other comp*/
	  if(cur_comp == in_component_array[j]) continue; 
          distance_ws(FK_inter[i], FK_inter[j], d);
          if (d<Connect_distance) {
           D[j_all].dist = d;
           D[j_all++].ind = j;
          }
         }
         qsort(D, j_all, sizeof(dpoint), dpoint_compare);
	 ON_PAPER(total_sorted += j_all;);

	 n_labels_i = 0; 	/*index to labels_i which holds the
		components with which a conf has been connected*/
	 jcount = my_MIN(heur_num, j_all);
	 for(j = tried = 0; (j<j_all) && (tried<jcount); j++) {
	  ON_PAPER(tried_connections++;);
	  if(Shrink) {
	   c_j = in_component_array[D[j].ind];
	   for(k=0,fl=1;(fl) && (k<n_labels_i); k++) 
	    if(c_j == labels_i[k]) fl = 0; 
	   if(!fl) continue;
	  }
   	  tried++;
          if(connect_simple(Inter_p[i], Inter_p[D[j].ind],
                                0, &path_len, NULL)) {
           set_store_str(Graph, i, D[j].ind, 1, CONNECT_LINE, 0, NULL);
	   if(Shrink) {labels_i[n_labels_i++] = c_j;}
	   ON_PAPER( suc_connections++;);
          }
         } /*for j*/
        } /* for i*/

	ON_PAPER(
	fprintf(fp, "AFTER:Total number of configurations sorted:    %d\n", 
			total_sorted);
	fprintf(fp, "AFTER:Total number of configurations tried:     %d\n", 
			tried_connections);
	fprintf(fp, "AFTER:Total number of configurations succeded:  %d\n", 
			suc_connections);
	fprintf(fp, "AFTER:Total connections %d, con/node = %f\n",
			suc_connections, (suc_connections*1.0)/(high_i-low_i));
	);
        printf("Try to connect (after) finished.\n");
}

/************************************************************************/

static  double w[MAX_INTER];	
static	double tmp[MAX_INTER];

int sel()
{
	double		x, drand48();
	int		i;

	x = drand48();
	i = 0;	
	while(w[i]<x) i++;
	return i;
}


int 	add_confs(estimate, fp)
int	estimate[MAX_INTER];
FILE	*fp;
{
	int 	i, j, c, selected, new_inter, path_len, suc_con = 0;
	double 	d;

	
		/*compute the weights af all nodes*/ 
	d = 0.0;
	for(i=0; i<No_inter; i++) { 
	 tmp[i] = 1.0/(estimate[i]+1.0);
	 d += tmp[i];
	}
	w[0] = tmp[0]/d;
	for(i=1; i<No_inter; i++) { 
	 w[i] = w[i-1] + tmp[i]/d;
	}
	w[No_inter] = 1.1;

	
		/*create M random points*/
	new_inter = No_inter;
	for(i=0; i<M; i++) {
	 selected = sel();

#if     DEBUG_PL
         XSetForeground(display,gc,WhitePixel(display,screen));
         XFillRectangle(display,win,gc,0,0,641,641);
         XFlush(display);
         XSetForeground(display,gc,BlackPixel(display,screen));
         draw_workspace(display, win, gc, screen, SCALE, SCALE,
                             ObjNum, ObjectwVertice, Objectw );
	 gets(s);
	 printf("Point %d Distance selected is %d\n", i, estimate[selected]);
         print_conf(Inter_p[selected]);
         draw_robot(Inter_p[selected], display, win, gc, screen, SCALE, SCALE);
	 printf("Selected has been draw.\n"); 
#endif

	 for(j=c=0; j<1 && c<No_inter; c = c+1) {
/*
          deviation_rand_conf(Inter_p[new_inter], Inter_p[selected],
		(int)(.1*Deviation*estimate[selected]));
*/
          deviation_rand_conf(Inter_p[new_inter], Inter_p[selected],
		Deviation);
          FK(Inter_p[new_inter], FK_inter[new_inter]);
          if(out_ws(FK_inter[new_inter]) ||
                VC(FK_inter[new_inter], Inter_p[new_inter]) ||
                auto_collision(Inter_p[new_inter], FK_inter[new_inter]))
          continue;

#if     DEBUG_PL
         print_conf(Inter_p[new_inter]);
         draw_robot(Inter_p[new_inter], display, win, gc, screen,SCALE,SCALE);
         gets(s);
#endif

	  In_component_array[new_inter] = new_inter; /*connect later*/
          init_store_str(&(Graph[new_inter]), 1);
	  if(Shrink) {
            /*try also to connect to have the same connected comps*/
           if(connect_simple(Inter_p[selected], Inter_p[new_inter],
                                 0, &path_len, NULL)) {
  	    suc_con++;
            set_store_str(Graph, selected, new_inter, 1, CONNECT_LINE,
                                 0, NULL);
            In_component_array[new_inter] = In_component_array[selected];
           }
	   else {
#if     DEBUG_PL
         print_conf(Inter_p[new_inter]);
         draw_robot(Inter_p[new_inter], display, win, gc, screen,SCALE,SCALE);
         gets(s);
#endif
	   }
	  } /*if shrink*/

          j++; new_inter++;
         }
	} /* for M loop*/

	ON_PAPER(fprintf(fp, "AUGMENT %d out of %d had suc connections\n", 
			suc_con, M);) 

	return new_inter;
}


/************************************************************************/

int 	add_confs_rw(estimate, fp)
int	estimate[MAX_INTER];
FILE	*fp;
{
	int 	i, j, c, selected, new_inter, len, suc_con = 0;
	int	up_bound;
	double 	d;
	static	double tmp[20000];
	static configuration tmp_path[1000];

		/*compute the weights af all nodes*/ 
	d = 0.0;
	for(i=0; i<No_inter; i++) { 
	 tmp[i] = 1.0/(estimate[i]+1.0);
	 d += tmp[i];
	}
	w[0] = tmp[0]/d;
	for(i=1; i<No_inter; i++) { 
	 w[i] = w[i-1] + tmp[i]/d;
	}
	w[No_inter] = 1.1;

	
		/*create M random points*/
	new_inter = No_inter;
	up_bound = (No_inter +M)*10;
	for(i=0; (new_inter<No_inter+M) && i<up_bound; i++) {
	 selected = sel();

	 len = 0;
	 random_bounce(Inter_p[selected], 500, &len, tmp_path); 
	 if(len==0) continue;

 	 assign_conf(tmp_path[len-1], Inter_p[new_inter]); 
	 FK(Inter_p[new_inter], FK_inter[new_inter]);
         init_store_str(&(Graph[new_inter]), 1);
	 set_store_str(Graph, selected, new_inter, 1, CONNECT_RPP_LINE,
                               len, tmp_path);

          new_inter++;
	} /* for M loop*/

	return new_inter;
}
