/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author: Jerome Barraquand, Tsai-Yen Li
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/



#include "defs.h"

extern  int  	construct_boundary();
extern  void 	compute_distance_map();
extern  void 	goal_connection();
extern  void 	skeleton_initialization();
extern  void 	skeleton_expansion();
extern  void 	set_configuration_space_dim();


#define DEBUG_GR		0   	/*if set, graphic vars declared*/
#define DEBUG_D1		0   	/*if set, graphic vars declared*/
#define DEBUG_DISTANCE_MAP 	0       /*see wave propagation*/
#define DEBUG_SKELETON		0       /*see the final skeleton*/ 
#define DEBUG_SKELETON_WS	0       /*see the skeleton in workspace*/ 
#define DEBUG_CON_SKELETON	0       /*see the connection of goal to sk*/ 
#define DEBUG_GR1		0   	/*if set, graphic vars declared*/
#define DEBUG_CSPACE		0

#if DEBUG_GR
		/* debugging display stuff */
	Display 	*display;
	int 		screen;
	Window 		win;
	GC 		gc;
	char 		s[10];
#endif

#if DEBUG_GR1
                /* debugging display stuff */
        Display         *display;
        int             screen;
        Window          win;
        GC              gc;
        char            s[10];
#endif



/************************************************************************
	Construct a map of the workspace form the algebraic equations
************************************************************************/

/* As it is right now this procedure should be called only at 
   the top, level - because of the number of parameters it 
   requires on the stack*/
void construct_wmap(status)
int	status[DIM][DIM];
{
	int		distance[DIM][DIM];
        IPoint  	boundary_wave[2][DIMFRONT];
        IdPoint 	origin[DIM][DIM];
        IPoint   	obj[MAXVERTEX];
        int             numOfBoundaryCells;
        int             i, j, xi, xj;

	memset(status, 0, DIM*DIM*sizeof(int));
	memset(distance, 0, DIM*DIM*sizeof(int));
        numOfBoundaryCells =
            construct_boundary(status, distance, boundary_wave, origin);
        for(i=0; i<ObjNum; i++) {
         /*build the obstacle*/
         for(j=0; j< ObjectwVertice[i]; j++)
          obj[j] = Objectw[j][i];
         fill_obstacle(status, obj, ObjectwVertice[i], 1);
        }

}



/***************************************************************************
	Construct the boundary of the obstacles, since these are input
	as polygons
****************************************************************************/

int construct_boundary(status, distance, boundary_wave, origin)
int status[DIM][DIM], distance[DIM][DIM];
IPoint boundary_wave[2][DIMFRONT];
IdPoint  origin[DIM][DIM];
{
  		/* raster variables */
  	int 	dx, dy, temp, interchange;
  	int 	x, y, xend, yend, e, s1, s2, k;
  	int 	i, j, vertexNum;
  	IPoint	dummy[MAXVERTEX];
  	int 	counter = 0;
  	int 	xt, yt;
  
  		/* loop over edges of all obstacles */
  	for(i=0; i<ObjNum; i++) {
    	 dummy[0].x = Objectw[0][i].x;
    	 dummy[0].y = Objectw[0][i].y;

    	 vertexNum = ObjectwVertice[i];
    	 for(j=1; j<=vertexNum; j++) {
      	  dummy[j].x = Objectw[j][i].x;
      	  dummy[j].y = Objectw[j][i].y;
      
      		/* start of raster algo */
      	  x = dummy[j-1].x;
      	  y = dummy[j-1].y;
      	  xend = dummy[j].x;
      	  yend = dummy[j].y;
      	  s1 = SIGN(xend - x);
      	  s2 = SIGN(yend - y);
      	  dx = my_ABS(xend - x);
      	  dy = my_ABS(yend - y);
      
      	  if(dy>dx) {
	   temp = dx;
	   dx = dy;
	   dy = temp;
	   interchange = 1;
      	  }
      	  else interchange = 0;
      
      	  e = (dy << 1) - dx;
      	  for(k=0; k<=dx; k++) {
	   xt = TRUNC(x);
	   yt = TRUNC(y);

		/* record pixel (xt, yt) as an obstacle, put it in the first
		   wave front.  */
	   status[xt][yt] = OBSTACLE;
	   boundary_wave[0][counter].x = xt;
	   boundary_wave[0][counter].y = yt;
	   counter++;
	   origin[xt][yt].x = xt;
	   origin[xt][yt].y = yt;
	   origin[xt][yt].id = i;   /*keep the id of the obstacle*/ 
	   distance[xt][yt] = 1;

	   while((e >= 0) && (dx > 0)) {
	    if( interchange ) x += s1;
	    else y += s2;
	    e -= dx << 1;
	   }
	
	   if( interchange ) y += s2;
	   else x += s1;
	
	   e += dy << 1;
      	  }
    	 }
	} 

  	return counter;
}

/*****************************************************************************
	Build the distance matrix.
*****************************************************************************/

void compute_distance_map(n, status, distance, boundary_wave, origin)
int n;
int status[DIM][DIM], distance[DIM][DIM];
IPoint boundary_wave[2][DIMFRONT];
IdPoint origin[DIM][DIM];
{
	int counter1, counter2;
	int x, y, dx, dy;
	int round, I, J;
	int d1;
	int i,j;

#if DEBUG_DISTANCE_MAP
	printf("\tDisplaying the distance map. ");
  	XSetForeground(display, gc, WhitePixel(display,screen));
  	XFillRectangle(display, win, gc, 0, 0, WIN_DIM+1, WIN_DIM+1);
  	XFlush(display);
  	XSetForeground(display,gc,BlackPixel(display,screen));
	for(i=0;i<DIM;i++) {
   	 for(j=0;j<DIM;j++) {
    	  if(status[i][j] == OBSTACLE) {
	   XDrawRectangle(display,win,gc,i*SCALE,j*SCALE,SCALE,SCALE);
	   XFillRectangle(display,win,gc,i*SCALE,j*SCALE,SCALE,SCALE);
          }
         }
	}
#endif

	counter1 = n;
	counter2 = 0;

	round = 1;
	I = (round + 1) % 2;
	J = (I + 1) % 2;

	while(counter1>0) {
  	 for(i=0; i<counter1; i++) {
          x = boundary_wave[I][i].x;
    	  y = boundary_wave[I][i].y;

#if DEBUG_DISTANCE_MAP
    	XFlush(display);
    	if(status[x][y] == VISITED) {
	 XSetForeground( display, gc, BlackPixel(display,screen) );
	 XDrawRectangle(display,win,gc,x*SCALE,y*SCALE,SCALE,SCALE);
	 XFillRectangle(display,win,gc,x*SCALE,y*SCALE,SCALE,SCALE);
    	}
    	if(status[x][y] == IN_SKELETON) {
	 XSetForeground( display, gc, BlackPixel(display,screen) );
	 XDrawRectangle(display,win,gc,x*SCALE,y*SCALE,SCALE,SCALE);
	 XFillRectangle(display,win,gc,x*SCALE,y*SCALE,SCALE,SCALE);
        }
#endif
    
	dx = x; dy = y;
	for(j = 0;j<4;j++) {
	 switch(j) {
	  case 0: dx = x + 1; dy = y; break;
	  case 1: dx = x - 1; break;
	  case 2: dx = x; dy = y + 1; break;
	  case 3: dy = y - 1; break;
	 }

	 if( dx >= DIM || dy >= DIM || dx < 0 || dy < 0 ) continue;
	 switch(status[dx][dy]) {
	  case OBSTACLE: break;
	  case IN_SKELETON: 
	  case VISITED:
	    	/*do not create a double skeleton*/
	   if(status[x][y] == IN_SKELETON) break; 
		/*quit if waves from the same obstacle meet*/
	   if (origin[x][y].id == origin[dx][dy].id) break;
		/*quit for lines formed due to 2 obstacles touching
		  each other*/
	   d1 = my_ABS( origin[dx][dy].x - origin[x][y].x )
	      + my_ABS( origin[dx][dy].y - origin[x][y].y );
	   if(d1 < 2) break;
	   status[dx][dy] = IN_SKELETON;
#if DEBUG_DISTANCE_MAP
 	   XSetForeground( display, gc, BlackPixel(display,screen) );
	   XDrawRectangle(display,win,gc,dx*SCALE,dy*SCALE,SCALE,SCALE);
	   XFillRectangle(display,win,gc,dx*SCALE,dy*SCALE,SCALE,SCALE);
	   XFlush(display);
#endif
	   break;
	  case FREE:
	   boundary_wave[J][counter2].x = dx;
	   boundary_wave[J][counter2].y = dy;
#if DEBUG_DISTANCE_MAP
	   XSetForeground( display, gc, BlackPixel(display,screen) );
	   XDrawRectangle(display,win,gc,dx*SCALE,dy*SCALE,SCALE,SCALE);
	   XFillRectangle(display,win,gc,dx*SCALE,dy*SCALE,SCALE,SCALE);
	   XFlush(display);
#endif
	   distance[dx][dy] = distance[x][y] + 1;
	   origin[dx][dy].x = origin[x][y].x;
	   origin[dx][dy].y = origin[x][y].y;
	   origin[dx][dy].id = origin[x][y].id;
	    
	   status[dx][dy] = VISITED;
	   counter2++;
	   break;
	  }
    	 }
	}
  	counter1 = counter2;
  	counter2 = 0;
  	round++;
  	I = (round + 1) % 2;
  	J = (I + 1) % 2;
	}

#if DEBUG_DISTANCE_MAP
	printf("Press <return>");
	gets(s);
#endif

#if DEBUG_SKELETON_WS 
	printf("\tDisplaying the skeleton in the workspace. ");
        XSetForeground(display, gc, WhitePixel(display,screen));
        XFillRectangle(display, win, gc, 0, 0, WIN_DIM+1, WIN_DIM+1);
        XFlush(display);
        XSetForeground(display,gc,BlackPixel(display,screen));
        draw_workspace(display, win, gc, screen, SCALE, SCALE, 
                                ObjNum, ObjectwVertice, Objectw);
	draw_skeleton(display, win, gc, screen, SCALE, SCALE, status);
	draw_skeleton(display, win, gc, screen, SCALE, SCALE, status);
	printf("Press <return>");
 	gets(s);
#endif

#if DEBUG_SKELETON
	printf("\tDisplaying the skeleton ONLY. ");
        XSetForeground(display, gc, WhitePixel(display,screen));
        XFillRectangle(display, win, gc, 0, 0, WIN_DIM+1, WIN_DIM+1);
        XFlush(display);
        XSetForeground(display,gc,BlackPixel(display,screen));
	draw_skeleton(display, win, gc, screen, SCALE, SCALE, status);
	draw_skeleton(display, win, gc, screen, SCALE, SCALE, status);
	printf("Press <return>");
 	gets(s);
#endif

}


	

/**************************************************************************
	Compute the L1 distance map
*************************************************************************/

	/*The following can be made more efficient.*/ 
compute_l1_distance(l1_distance_map)
int l1_distance_map[DIM][DIM];
{
	int      status[DIM][DIM];
        int      numOfBoundaryCells;
        IPoint   boundary_wave[2][DIMFRONT];
        IdPoint  origin[DIM][DIM];

#if DEBUG_D1
        display = XOpenDisplay(NULL);
        screen = DefaultScreen(display);
        gc = XCreateGC(display, RootWindow(display,screen), 0, NULL);
        win = XCreateSimpleWindow(display, RootWindow(display, screen),
                            0,0,640,640,DefaultDepth(display,screen),
                            BlackPixel(display, screen),
                            WhitePixel(display, screen));
        XSelectInput(display, win, ExposureMask | ButtonPressMask);
        XMapWindow(display,win);
        XFlush(display);
        printf("\tPress <return> for graphics.");
        gets(s);
#endif


	memset(status, 0, DIM*DIM*sizeof(int));
        memset(l1_distance_map, 0,  DIM*DIM*sizeof(int));
        numOfBoundaryCells =
          construct_boundary(status, l1_distance_map, boundary_wave, origin);
        compute_distance_map(numOfBoundaryCells, status, l1_distance_map,
          boundary_wave, origin);
}
	


/**************************************************************************
	Constuct the Cspace potential for one of the links
 * set_configuration_space_dimension :
 * This function calculates the number of steps along the theta axis.
 * The number depends on the size of the bar and of the size of a cell in the
 * workspace.
 * Basically it is such that for 1 increment or decrement of theta, no point
 * of the bar will move across more than 1 cell.
 * This is done to avoid jumping obstacles without bumping into them.
**************************************************************************/
void set_configuration_space_dim(RobotVertice, Robot, dth, dtheta)
int 	RobotVertice;
IPoint 	Robot[MAX_LINK_VERTICES];
int 	*dth;
double 	*dtheta;
{
  	int 	i;
  	double 	a, b, r;
  
  	r = 0;
  	for(i=0;i<RobotVertice;i++) {
    	 a = (Robot[i].x) * (Robot[i].x) * 1.0;
    	 b = (Robot[i].y) * (Robot[i].y) * 1.0;
    	 r = my_MAX(r, sqrt( a + b ));
  	}
  
  	*dtheta = ( 1.0/ r);
  	*dth = ceil( 2 * M_PI / (*dtheta) );
  	if( (*dth) >= DIM ) { 
    	 printf("Grid (%d) too small for theta.\n",*dth);
         exit(1); 
  	}
/*      temporary change*/
	*dth = DIM;
	*dtheta = (2.0*M_PI) / DIM;
  	printf("Cspace dimensions 128x128x%d\n",*dth);
}  
  

void construct_cspace(c_space, RobotVertice, Robot, dth, dtheta)
int     c_space[DIM][DIM][DIM/32];
int     RobotVertice;
IPoint 	Robot[MAX_LINK_VERTICES];
int 	dth;
double  dtheta;

{ /* variables of bitmap filling */

  	double 		pimul2 = 2 * M_PI, pidiv2 = M_PI / 2;
  	double 		c, s;
  	double 		alpha1, alpha, delta, diff;
  	int 		obstindex, robotindex;
  	int 		cobstvert;
  	int  		flag;
	struct {
	 double x, y;
	} 		rotrobvert[MAXVERTEX];
	double 		robnegnorm[MAXVERTEX];
	double 		rotrobnegnorm[MAXVERTEX];
	double 		obstnorm[MAXVERTEX][MAXOBST];
	double 		betaobst[MAXVERTEX], betarob[MAXVERTEX];
	IPoint 		dummy[MAXVERTEX];
	int 		i, j, j1, k1, k, t;
	int		slice_ind, obs_ind, cobs_ind, vertexNum;
	int 		dx, dy, temp, interchange;
	int 		x, y, xend, yend, e, s1, s2;
	int 		bplane, bpos;
	int 		xt0, xt1, xt2, yt0, yt1, yt2, ii, jj;
	int 		tmp_cspace[DIM][DIM];
	int		count = 0;
	char 		st[10];
	
#if DEBUG_GR
        display = XOpenDisplay(NULL);
        screen = DefaultScreen(display);
        gc = XCreateGC(display, RootWindow(display,screen), 0, NULL);
        win = XCreateSimpleWindow(display, RootWindow(display, screen),
                            0,0,WIN_DIM,WIN_DIM,DefaultDepth(display,screen),
                            BlackPixel(display, screen),
                            WhitePixel(display, screen));
        XSelectInput(display, win, ExposureMask | ButtonPressMask);
        XMapWindow(display,win);
        XFlush(display);
        printf("\tPress <return> for graphics.");
        gets(s);
#endif

	  	/* reset the configuration space */
	  
	memset(c_space, 0, DIM*DIM*(DIM/32)*sizeof(int));
	  
	   	/* precomputation of the normals of the obstacles and
	   	 * of the negated normals of the robot.
	    	 * necessary to calculate the vertices of the c-obstacles.*/
	  
	  	/* calculates negated normals of robot */
	for(i=0; i<RobotVertice; i++ ) {
	 c = (Robot[i+1].x - Robot[i].x)/(1.0*DIM);
	 s = (Robot[i+1].y - Robot[i].y)/(1.0*DIM);
	   
	 robnegnorm[i] = atan2(s, c) + pidiv2;
	   
	 if( robnegnorm[i] > pimul2 ) robnegnorm[i] -= pimul2;
	 if( robnegnorm[i] < 0 ) robnegnorm[i] += pimul2;
	}
	
	 	 /* calculates normals of obstacles */
	for(i=0; i<ObjNum; i++) {
	 vertexNum = ObjectwVertice[i];
	 for(j = 0; j < vertexNum; j++) {
	  c = (Objectw[j+1][i].x - Objectw[j][i].x);
	  s = (Objectw[j+1][i].y - Objectw[j][i].y);
	      
	  obstnorm[j][i] = atan2( s, c ) - pidiv2;
	     
	  if( obstnorm[j][i] < 0 ) obstnorm[j][i] += pimul2;
	  if( obstnorm[j][i] > pimul2 ) obstnorm[j][i] -= pimul2;
	 }
	}
	
	   	/* loop along the theta axis
	   	 * to calculate every slice of the configuration space.*/
	  
	for(slice_ind=0; slice_ind<(dth); slice_ind++) {
#if DEBUG_CSPACE
	  printf("computing configuration space slice = %d\n", slice_ind);
	  XSetForeground(display,gc,WhitePixel(display,screen));
	  XFillRectangle(display,win,gc,0,0,WIN_DIM+1,WIN_DIM+1);
	  XFlush(display);
#endif
	 alpha1 = dtheta*slice_ind;
	 c = cos(alpha1);
	 s = sin(alpha1);
	    
	  	 /* calculates which bit plane and which bit
	     	  * because the c-space array is packed.
	     	  * every position is described by 1 bit.*/
	 bplane = slice_ind >> 5;
	 bpos = 1 << (slice_ind - (bplane << 5));
	    
	    	 /* calculates the rotated normals of the robot
	     	  * thus taking into account the rotation.*/
	 for( k = 0; k < RobotVertice; k++ ) {
	  rotrobvert[k].x = c * ( Robot[k].x)
	       - s * ( Robot[k].y);
	  rotrobvert[k].y = s * ( Robot[k].x)
	       + c * ( Robot[k].y);
	
	  rotrobnegnorm[k] = robnegnorm[k] + alpha1;
	   if(rotrobnegnorm[k] > pimul2) rotrobnegnorm[k] -= pimul2;
	 }
	
	  	  /* loop over the obstacles */
	 t = 0;
	 for(obs_ind = 0; obs_ind < ObjNum; obs_ind++) {
	      
	     /* locates the obstacle normal just before the first negated 
	      *  normal of the robot which angle is alpha*/
	  alpha = rotrobnegnorm[0];
	  delta = 7.;
	      
	  vertexNum = ObjectwVertice[obs_ind];
	  obstindex = 0;
	  for( k = 0; k < vertexNum; k++ ) {
	   betaobst[k] = obstnorm[k][obs_ind] - alpha;
	   if(betaobst[k] < 0) betaobst[k] += pimul2;
	
	   diff = obstnorm[k][obs_ind] - alpha;
	   if(diff < 0) diff += pimul2;
	   if(diff < delta) {
	    delta = diff;
	    obstindex = k;
	   }
	  }
	      
	  robotindex = 0;
	  flag = 1;
	      /*offset the angles of the negated normals by the quantity -alpha,
	       * setting the robot negated normal with respect to the x-axis.
	       */
	  for( k = 0; k < RobotVertice; k++ ) {
	   betarob[k] = rotrobnegnorm[k] - alpha;
	   if(betarob[k] < 0) betarob[k] += pimul2;
	  }
	      
	    	  /* calculates the vertices of the c_obstacle */
	  dummy[0].x = GRID( Objectw[obstindex][obs_ind].x
				- rotrobvert[(robotindex+1) % RobotVertice].x );
	  dummy[0].y = GRID( Objectw[obstindex][obs_ind].y
				- rotrobvert[(robotindex+1) % RobotVertice].y );
	  cobstvert = ObjectwVertice[obs_ind] + RobotVertice;
	  for(cobs_ind = 1; cobs_ind < cobstvert; cobs_ind++) {
   	   if(flag==1) {
	    betarob[robotindex] += pimul2;
  	    robotindex = ( robotindex + 1 ) % RobotVertice;
		  
	    if(betarob[robotindex] >= betaobst[obstindex] ) flag = 0;
	   }
	   else {
	    betaobst[obstindex] += pimul2;
	  
	    obstindex = ( obstindex + 1 ) % ObjectwVertice[obs_ind];
  	    if( betaobst[obstindex] > betarob[robotindex] ) flag = 1;
	   }
		
	   if( flag == 1 ) {
	    dummy[cobs_ind].x = GRID( Objectw[obstindex][obs_ind].x 
					   - rotrobvert[(robotindex+1) %
							RobotVertice].x );
	    dummy[cobs_ind].y = GRID( Objectw[obstindex][obs_ind].y
					   - rotrobvert[(robotindex+1) %
							RobotVertice].y );
	   }
	   else {
	    dummy[cobs_ind].x = GRID( Objectw[(obstindex+1) %
					   ObjectwVertice[obs_ind]][obs_ind].x
					   - rotrobvert[robotindex].x );
	    dummy[cobs_ind].y = GRID( Objectw[(obstindex+1) %
					   ObjectwVertice[obs_ind]][obs_ind].y
					   - rotrobvert[robotindex].y );
	   } 
	  } /* for (cobs_ind=..) */
	      
	  dummy[cobstvert].x = dummy[0].x;
	  dummy[cobstvert].y = dummy[0].y;
	      
	  memset(tmp_cspace, 0, DIM*DIM*sizeof(int));
	     	 /* the edges of the c-obstacle are blackened there after.
	       	  * a rasterizing technic is used for that purpose.*/
	  for(j1 = 0; j1<cobstvert; j1++) {
		
			/* start of raster algo */
  	   x = dummy[j1].x;
	   y = dummy[j1].y;
	   xend = dummy[j1+1].x;
	   yend = dummy[j1+1].y;
	   s1 = SIGN(xend - x);
	   s2 = SIGN(yend - y);
	   dx = my_ABS(xend - x);
	   dy = my_ABS(yend - y);
		
	   if(dy > dx) {
	    temp = dx;
	    dx = dy;
	    dy = temp;
	    interchange = 1;
	   }
	   else interchange = 0;
	   e = (dy << 1) - dx;
		
	   for( k1 = 0; k1 <= dx; k1++ ) {
	    xt1 = TRUNC(x);
	    yt1 = TRUNC(y);
	    xt0 = TRUNC(x-1);
	    yt0 = TRUNC(y-1);
	    xt2 = TRUNC(x+1);
	    yt2 = TRUNC(y+1);
	  
		  /* boundary of obstacle, thickness = 1 */
	    tmp_cspace[xt1][yt1] = 1; 
#if DEBUG_CSPACE
	if(slice_ind==1112 || slice_ind ==1119) {
  	     XSetForeground(display,gc,BlackPixel(display,screen));
	     XDrawRectangle(display,win,gc,xt1*SCALE,yt1*SCALE,SCALE,SCALE);
	     XFillRectangle(display,win,gc,xt1*SCALE,yt1*SCALE,SCALE,SCALE);
	}
#endif	  	  
		  /* thickening the boundary according to interchange */
/*
	    if(interchange == 0) {
	     tmp_cspace[xt1][yt0] = tmp_cspace[xt1][yt2] = 1; 
#if DEBUG_CSPACE
	     if(slice_ind%10==0) {
	      XSetForeground(display,gc,BlackPixel(display,screen));
	      XDrawRectangle(display,win,gc,xt1*SCALE,yt0*SCALE,SCALE,SCALE);
	      XFillRectangle(display,win,gc,xt1*SCALE,yt0*SCALE,SCALE,SCALE);
	      XDrawRectangle(display,win,gc,xt1*SCALE,yt2*SCALE,SCALE,SCALE);
	      XFillRectangle(display,win,gc,xt1*SCALE,yt2*SCALE,SCALE,SCALE);
	      XFlush(display);
	     }
#endif	  
	    }
	    else {
	     tmp_cspace[xt0][yt1] = tmp_cspace[xt2][yt1] = 1; 
#if DEBUG_CSPACE
	     if(slice_ind%10==0) {
              XSetForeground(display,gc,BlackPixel(display,screen));
	      XDrawRectangle(display,win,gc,xt0*SCALE,yt1*SCALE,SCALE,SCALE);
	      XFillRectangle(display,win,gc,xt0*SCALE,yt1*SCALE,SCALE,SCALE);
	      XDrawRectangle(display,win,gc,xt2*SCALE,yt1*SCALE,SCALE,SCALE);
	      XFillRectangle(display,win,gc,xt2*SCALE,yt1*SCALE,SCALE,SCALE);
	      XFlush(display);
	     }
#endif	  
	    }
*/
	    while((e >= 0)  && (dx > 0)) {
	     if(interchange) x += s1;
	     else y += s2;
	     e -= dx << 1;
	    }
		 
	    if(interchange) y += s2;
	    else x += s1;
		  
	    e += dy << 1;
	   } /* end of for k1... */
	  } /* end of for j1... */
	  		/*to avoid problems with collision checking we
			  fill the C-obstacles here. We cheat by taking into
	 		  account the info of the workspace obstacle to 
			  make the filling procedure more easy*/ 

	  t += fill_obstacle(&(tmp_cspace[0][0]), &(dummy[0]), cobstvert,
				slice_ind); 
	 
			/*copy the array to the bitmap, unfortunately 
			  this has to be done for every object, should change*/
	  count = 0;
	  for(ii=0; ii<DIM; ii++) 
	   for(jj=0; jj<DIM; jj++) 
	    if(tmp_cspace[ii][jj]) {
	     c_space[ii][jj][bplane] = c_space[ii][jj][bplane] | bpos; 
	     count++;
	    }
#if DEBUG_CSPACE
	  if(slice_ind ==1112 || slice_ind == 1119) {
           for(ii=0; ii<DIM; ii++)
            for(jj=0; jj<DIM; jj++)
             if(tmp_cspace[ii][jj]) {
              XSetForeground(display,gc,BlackPixel(display,screen));
              XDrawRectangle(display,win,gc,ii*SCALE,jj*SCALE,SCALE,SCALE);
              XFlush(display);
             }
	    printf("<Press return>"); gets(st);
	   }
#endif

			/*two ckecks to make sure the fill of the
			  obstacles has been done correctly*/ 
	  if(count>DIM*40) {
	   printf("Make sure the free space is not completely filled.\n");
	   printf("C-space obtacle appears very big.\n");
#if DEBUG_CSPACE
          XSetForeground(display,gc,WhitePixel(display,screen));
          XFillRectangle(display,win,gc,0,0,WIN_DIM+1,WIN_DIM+1);
          XFlush(display);
	   for(ii=0; ii<DIM; ii++) 
	    for(jj=0; jj<DIM; jj++) 
	     if(tmp_cspace[ii][jj]) {
              XSetForeground(display,gc,BlackPixel(display,screen));
              XDrawRectangle(display,win,gc,ii*SCALE,jj*SCALE,SCALE,SCALE);
              XFlush(display);
	     }
#endif
	   printf("<Press return>"); gets(st);
	  }
	 } /* end of for obs_ind... */

			/*make sure all obstacles have been filled*/
         if(t<=6)  {
	  printf("Possible problem at theta %d, (obst filled %d, not %d )\n", 
					slice_ind, t, ObjNum);
#if DEBUG_CSPACE
          XSetForeground(display,gc,WhitePixel(display,screen));
          XFillRectangle(display,win,gc,0,0,WIN_DIM+1,WIN_DIM+1);
          XFlush(display);
           for(ii=0; ii<DIM; ii++)
            for(jj=0; jj<DIM; jj++)
             if(c_space[ii][jj][bplane] & bpos) {
              XSetForeground(display,gc,BlackPixel(display,screen));
	      XDrawRectangle(display,win,gc,ii*SCALE,jj*SCALE,SCALE,SCALE);
              XFlush(display);
             }
           printf("<Press return>"); gets(st);
#endif
          }
	} /* end of for slice_ind... */

#if DEBUG_GR
        XFlush(display);
        XCloseDisplay(display);
#endif

}
	

	
int fill_obstacle(cspace_slice, obstacle, cobstvert, slice_ind)
int 	cspace_slice[DIM][DIM];
IPoint 	obstacle[MAXVERTEX];
int 	cobstvert, slice_ind;
{
	int		i, j, x, y, index_front, index_back, found ;
	int		s1, s2, dx, dy, temp, interchange, e;
	static IPoint   list[DIM*DIM];
	static int	visited[DIM][DIM];	


	memset(visited, 0, DIM*DIM*sizeof(int));
		/*find a point inside the obstacle, cheating a little 
		  bit here, because obstacles are convex. Just a 
		  heuristic*/
	if(cobstvert<2) {
	 printf("Cspace construction can not deal with obsts with %d verts\n", 
				cobstvert);
	 exit(-1);
	}
		/*find a point inside the obstacle, cheat a bit because
		  we know that the obstacle is convex*/
	found = 0;
		/*use a very simple algo and see if it works*/
        x = y = 0;
        for(i=0; i<cobstvert; i++) {
         x += obstacle[i].x;
         y += obstacle[i].y;
        }
        x = TRUNC( GRID(x/(1.0*cobstvert)));
        y = TRUNC( GRID(y/(1.0*cobstvert)));
        if(!cspace_slice[x][y]) found = 1;
	
		/*try a more complicated algo for very thin obstacles*/
	for(i=2; (i<cobstvert) && (!found); i++) {
	 x = obstacle[0].x;
	 y = obstacle[0].y;
	 s1 = SIGN(obstacle[i].x - x);
	 s2 = SIGN(obstacle[i].y - y);	
	 dx = my_ABS(obstacle[i].x - x);
	 dy = my_ABS(obstacle[i].y - y);
	 if(dy>dx) {
	  temp = dx; dx = dy; dy = temp; 
	  interchange = 1;
	 }
	 else interchange = 0;
	 e = (dy << 1) - dx;
	 for(j=0; (j<dx) && (!found); j++) {
	  while(e>=0 && dx>0) {
	   if(interchange==1) x = x + s1;
	   else y = y + s2;
	   e = e - (dx << 1);
	  }
	  if(interchange==1) y = y + s2;
	  else x = x + s1;
	  e = e + (dy << 1);
	  if((x>=0 && y>=0 && x<DIM && y<DIM) && (!cspace_slice[x][y])) 
	   found = 1;
	 } 
	}
	
			/*now fill the obstacle, if a point inside 
			  the obstacle has been found*/
	if(found) { 
		/*fill the obstacle starting from the point found*/
	
	 index_front = 0;
	 list[0].x = x; 
	 list[0].y = y;
	 visited[x][y] = 1;
	 index_back = 1;
	
	 while(index_front!=index_back) {
	  x = list[index_front].x; 
	  y = list[index_front++].y; 
	  cspace_slice[x][y] = 1;
 
	  if((x<DIM-1)&&(!cspace_slice[x+1][y])&&(!visited[x+1][y])){
	   list[index_back].x = x+1;
	   list[index_back++].y = y;
	   visited[x+1][y] = 1;
	  }
	  if((x>1)&&(!cspace_slice[x-1][y])&&(!visited[x-1][y])) {
	   list[index_back].x = x-1;
           list[index_back++].y = y;
	   visited[x-1][y] = 1;
          }
	  if((y<DIM-1)&&(!cspace_slice[x][y+1])&&(!visited[x][y+1])) { 
	   list[index_back].x = x;
           list[index_back++].y = y+1;
	   visited[x][y+1] = 1;
	  }
	  if((y>1)&&(!cspace_slice[x][y-1])&&(!visited[x][y-1])) {
	   list[index_back].x = x;
	   list[index_back++].y = y-1;
	   visited[x][y-1] = 1;
	  }
	  if(index_front>index_back)
	   printf("index_front %d index_back %d\n", index_front, index_back);
	 }
	}
	return found;
}

