/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include "defs.h"


#define DEBUG_GR		0   	/*if set, graphic vars declared*/
#define DEBUG_Wspace 		0	/*for cspace debugging*/
#define DEBUG_Robot  		0 	/*for robot debugging */

extern	int	illegal_conf();

#if DEBUG_GR
		/* debugging display stuff */
	Display 	*display;
	int 		screen;
	Window 		win;
	GC 		gc;
	char 		s[10];
#endif

/****************************************************************************
	preprocess:
	Data is strored in the files given in the args of the procedure.
	If no names have been specified in the command line the defaults
	are:
	the distance map in the file dist.dat
        the heuristic potentials in files pot.dat
        and the cspace  in file cspace.dat
****************************************************************************/
void preprocess(fname, cspace_fname, dist_fname)
char 	*fname;
char 	*cspace_fname, *dist_fname;
{
        time_t          t1, t2;
	int		i, j;
	double		thetaDim_incr;
	FPoint		ws_pos[NBP];
	char		new_name[100], *strcat();

#if DEBUG_GR
	display = XOpenDisplay(NULL);
  	screen = DefaultScreen(display);
  	gc = XCreateGC(display, RootWindow(display,screen), 0, NULL);
  	win = XCreateSimpleWindow(display, RootWindow(display, screen),
			    0,0,WIN_DIM,WIN_DIM,DefaultDepth(display,screen), 
			    BlackPixel(display, screen),
			    WhitePixel(display, screen));
  	XSelectInput(display, win, ExposureMask | ButtonPressMask);
  	XMapWindow(display,win);
  	XFlush(display);
	printf("\tPress <return> for graphics.");
	gets(s);
#endif


	/************** read the workspace or not********************/
	if(Read_Workspace) {
	 my_get_exact_time(t1);
	 read_obstacles_robot(fname, &ObjNum, ObjectwVertice, Objectw, 
			     Begin_pos, End_pos,
			     &Num_cntl_points, Cntl_points, 
			     &Link_length, &Link_vertices, Link, 
			     &BaseX, &BaseY);
	 my_get_exact_time(t2);
	 printf("Workspace Data Read in %f\n", (t2-t1)/TICKS_PER_SECOND);
	/*construct a bitmap of the workspace*/
	}
	construct_wmap(Wspace);

#if DEBUG_Wspace
	printf("\tDisplaying the Workspace. ");
  	XSetForeground(display, gc, WhitePixel(display,screen));
  	XFillRectangle(display, win, gc, 0, 0, WIN_DIM+1, WIN_DIM+1);
  	XFlush(display);
  	XSetForeground(display,gc,BlackPixel(display,screen));
  	draw_workspace(display, win, gc, screen, SCALE, SCALE, 
				ObjNum, ObjectwVertice, Objectw);
	printf("Press <return> for the bitmap output");
  	gets(s);
	XSetForeground(display, gc, WhitePixel(display,screen));
        XFillRectangle(display, win, gc, 0, 0, WIN_DIM+1, WIN_DIM+1);
        XFlush(display);
        XSetForeground(display,gc,BlackPixel(display,screen));
        draw_workspace_map(display, win, gc, screen, SCALE, SCALE,
			Wspace);
	printf("Press <return> ");
	gets(s);
#endif

	/************compute lookup tables for sin and cos**************/
        my_get_exact_time(t1);
        for(i=0; i<DIM; i++) {
         Table_sin[i] = sin((i*2*M_PI)/DIM);
         Table_sin_length[i] = Link_length*Table_sin[i];
         Table_cos[i] = cos((i*2*M_PI)/DIM);
         Table_cos_length[i] = Link_length*Table_cos[i];
        }
        my_get_exact_time(t2);
        printf("Took %f sec to compute the lookup tables.\n",
                                (t2-t1)/TICKS_PER_SECOND);

	/**************deal with the distance construction************/
        if(Use_preprocess)
         read_distance(dist_fname, Dist, &Dim);
        else 
         compute_l1_distance(Dist);

	for(i=0; i<DIM; i++) 
	 for(j=0; j<DIM; j++) {
	  if(Wspace[i][j]>0) Dist[i][j] = -1;
	 }

	/**********deal with cspace construction**********************/

	if(Use_preprocess) { 
	 read_cspace(cspace_fname, Cspace, &Dim, &ThetaDim);
	}
	else {

	   /*construct the Cspace used for all links but the base*/ 
	 printf("Constructing Cspace for the links.....\n");
	 read_obstacles_robot(fname, &ObjNum, ObjectwVertice, Objectw,
                             Begin_pos, End_pos,
                             &Num_cntl_points, Cntl_points,
                             &Link_length, &Link_vertices, Link,
                             &BaseX, &BaseY);
	 my_get_exact_time(t1);
	 set_configuration_space_dim(Link_vertices, Link, 
				&ThetaDim, &thetaDim_incr);
	 construct_cspace(&(Cspace[0][0][0]), Link_vertices, Link, 
				ThetaDim, thetaDim_incr);
	 my_get_exact_time(t2);
	 printf("C-space constructed in %f sec\n", 
				(t2-t1)/TICKS_PER_SECOND);
	} 

	/**********deal with the initial and final position************/
	if(Check_init_positions) {
	 int	state[3];
	
	 if(illegal_conf(Begin_pos, ws_pos, state)) {
	  printf("Initial configuration is invalid: O[%d], A[%d], VC[%d]\n", 
			state[0], state[1], state[2]);
	  exit(-1);
	 }
	 if(illegal_conf(End_pos, ws_pos, state)) {
	  printf("Final configuration is invalid: O[%d], A[%d], VC[%d]\n", 
                        state[0], state[1], state[2]);
	  print_conf(End_pos);
          exit(-1);
         }
	}
	
	/***********write data if asked *******************************/
	if(!Use_preprocess) {
	 write_cspace(cspace_fname, Cspace, Dim, ThetaDim);
         write_distance(dist_fname, Dist, Dim);
        }

#if DEBUG_GR
        XFlush(display);
        sleep(1);
        XCloseDisplay(display);
#endif
}


int illegal_conf(conf, ws_pos, state)
configuration 	conf;
FPoint		ws_pos[NBP];
int		state[3];
{

	state[0] = state[1] = state[2] = 0;
	FK(conf, ws_pos);
        if(out_ws(ws_pos)) {
	 state[0] = 1; 
	 return 1;
        }
        if(auto_collision(conf, ws_pos)) {
	 state[1] = 1;
	 return 1;
        }
	if(VC(ws_pos, conf)) {
	 state[2] = 1;
	 return 1;
        }
	return 0;
}

	 
