/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>

#include "defs.h"
#include "extern.h"

#include "globals.c"

#define		NUM_PAIRS	50
char		Pairs_file[100];
int		Random_test = 0;

			/*for printing global data*/
data		Gl_stats[] = {
{"", NULL, NEW_LINE},
{"Path found", &Found, BOOLEAN}, 
{"   Time for actual planning", &Timings[0], REAL},
{"      Time to connect init conf", &Timings[1], REAL}, 
{"      Time to search the graph", &Timings[2], REAL}, 
{"      Time to connect final conf", &Timings[3], REAL}, 
{"   Time for smoothing ", &Timings[6], REAL},
{"", NULL, NEW_LINE},
{"Path length before smoothing", &Lengths[0], INTEGER},  
{"      For init conf connection", &Lengths[1], INTEGER}, 
{"      In the network", &Lengths[2], INTEGER},  
{"      For final conf connection", &Lengths[3], INTEGER}, 
{"Path length after smoothing", &Lengths[6], INTEGER}, 
{"", NULL, NEW_LINE},
{"Input workspace", Input_file_name, STRING},
{"Input network ", Input_network_name, STRING},
{"Initial configuration", Begin_pos, CONFIGURATION}, 
{"Final configuration", End_pos, CONFIGURATION}, 
{"Network configurations", &No_inter, INTEGER}, 
{"", NULL, END_GLOBALS}
};	


		/*procedure for obtaining additional arguments in the 
		  command line*/
void read_args(argc, argv, use_preprocess, cspace_fname, dist_fname, 
	        input_network_name, workspace_name,
		seed, path_file_name, connect_distance, 
		random_connect_length)
int argc;
char *argv[];
int *use_preprocess, *seed;
int *connect_distance, *random_connect_length;
char *cspace_fname, *dist_fname, *input_network_name, *workspace_name;
char *path_file_name;
{
	int i;
        for(i=0; i<argc; i++) {
	  if(strcmp(argv[i], "-P")==0)
	   *use_preprocess = atoi(argv[i+1]);
	  if(strcmp(argv[i], "-CF")==0)
	   cspace_fname = strcpy(cspace_fname, argv[i+1]);
	  if(strcmp(argv[i], "-DF")==0)
           dist_fname = strcpy(dist_fname, argv[i+1]);
	  if(strcmp(argv[i], "-NF")==0)
           input_network_name = strcpy(input_network_name, argv[i+1]);
	  if(strcmp(argv[i], "-WF")==0)
           workspace_name = strcpy(workspace_name, argv[i+1]);
	  if(strcmp(argv[i], "-S")==0)
	   *seed = atoi(argv[i+1]);
	  if(strcmp(argv[i], "-O")==0)
           path_file_name = strcpy(path_file_name, argv[i+1]);
	  if(strcmp(argv[i], "-DD")==0)
	   *connect_distance= atoi(argv[i+1]);
	  if(strcmp(argv[i], "-RL")==0) 
	   *random_connect_length = atoi(argv[i+1]);
	}
}

void main(argc,argv)
int argc;
char *argv[];
{
	int 		seed = 47326;
	FILE		*fp;

	if(argc<3) {
 	 printf("USE:interf <stat_f><query_time(sec)>\n");
	 printf("Optional parameters: \n");
	 printf("\t -P  <1 to use  preprocessing>  (default = 0)\n");
	 printf("\t -CF <cspace file>		(default = cspace.dat)\n");
	 printf("\t -DF <dist  file>		(default = dist.dat)\n");
	 printf("\t -NF <network file>		(default = none)\n");
	 printf("\t -WF <workspace file>	(default = none)\n");
	 printf("\t -S  <seed>			(default = 1994)\n");	
	 printf("\t -0  <path filename>		(default = none)\n");	
	 printf("\t -DD <connect distance> 	(no default)\n");
	 printf("\t -RL <random_connect_len> 	(no default)\n");
	 exit(-1);
	}
	strcpy(Stat_file, argv[1]);
	sprintf(Cspace_fname, "cspace.dat");
	sprintf(Dist_fname, "dist.dat");

	Connect_distance = BIG_DIST;
		/*establish input parameters*/
	read_args(argc, argv, &Use_preprocess, Cspace_fname,  
			Dist_fname, Input_network_name, Workspace_name, 
			&seed, Path_file_name, &Connect_distance, 
			&Random_connect_length);
		/*initialize random number generator*/
	srandom(seed + 10);

	Query_time = atoi(argv[2])*TICKS_PER_SECOND;
	With_interface = 1; 

        main_motif(argc, argv);

}

