/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include "defs.h"

extern	int	isdigit();
/**************************************************************************
	Routines for reading and writing files that contain 
	information relevant to the planner.
	Notice that the dimension of the discretization of 
	the workspace is always written and read from these
	files for error checking purposes.
**************************************************************************/


/**************************************************************************
		Read the obstacles from the environment file
**************************************************************************/
	
void read_obstacles_robot(fname, objnum, objectvertice, object,
				    begin_pos, end_pos, 
				    num_cntl_points, cntl_points,
				    link_length, link_vertices, link,
				    basex, basey)
char 	*fname;
int 	*objnum; 
int 	objectvertice[MAXOBST];
IPoint	object[MAXVERTEX][MAXOBST];
configuration begin_pos, end_pos;
int 	*num_cntl_points;
int 	cntl_points[NUM_CNTL_POINTS];
int 	*link_length;
int 	*link_vertices;
IPoint 	link[MAX_LINK_VERTICES]; 
int 	*basex, *basey;
{
	FILE 	*fp;
	int	dimread, dofread, nbpread, len ; 
	int 	i, tch, num, dummy_x, dummy_y;
	double 	tmp;

	ON_COMMENTS(printf("Reading input file %s.\n", fname);)
	if(!(fp = fopen(fname,"r"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
	 exit(-1);
        }

	tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
	fscanf(fp, " %d", &dimread);
	printf("Dim read is %d\n", dimread);
        if(dimread!=DIM) {
         printf("Dim of the workspace read(%d), is not the expected one.\n",
		dimread);
         printf("Change DIM in the \"defs.h\" file and recomplile.\n");
         exit(-1);
        }

		/*skip initial junk from the input file*/
	tch = getc(fp);
	tch = getc(fp);
	while (tch != '\n') tch = getc(fp);
	tch = getc(fp);

			/*read the obstacles first, notice that  
			  the chain of vertices should be closed*/
	*objnum = 0;
	while(isdigit(tch)) {
  	 ungetc(tch, fp);
  	 fscanf(fp, " %d", &num); 

	 objectvertice[*objnum] = num;
  	 for(i=0; i<num; i++) {
          fscanf(fp, " %d, %d", &dummy_x, &dummy_y);

	  object[i][*objnum].x = dummy_x;
          object[i][*objnum].y = dummy_y;

          if(i==0) {
           object[num][*objnum].x = dummy_x;
           object[num][*objnum].y = dummy_y;
          }
         }
         (*objnum)++;

	 tch = getc(fp);
	 while (tch != '\n') tch = getc(fp);
	 tch = getc(fp);
	}

		/*read and check the DOF*/
        while (tch != '\n') tch = getc(fp);
	fscanf(fp, " %d", &dofread);
	printf("DOF read is %d\n", dofread);
        if(dofread!=DOF) {
         printf("DOF read(%d) is not the expected one.\n", dofread);
         printf("Change DOF in the \"defs.h\" file and recomplile.\n");
         exit(-1);
        }

		/*read and check the NBP*/
	tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
	fscanf(fp, " %d", &nbpread);
	printf("NBP read is %d\n", nbpread);
	if(nbpread!=NBP) {
         printf("NBP  read (%d)is not the expected one.\n", nbpread);
         printf("Change NBP in the \"defs.h\" file and recompile.\n");
	 exit(-1);
	}

		/*read the description of each link of the robot*/
        tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
        tch = getc(fp);

        ungetc(tch, fp);
        fscanf(fp, " %d", link_vertices);
	fscanf(fp, " %d", link_length);

        for(i=0; i<*link_vertices; i++) {
         fscanf(fp, " %d, %d", &dummy_x, &dummy_y);
	 link[i].x = dummy_x;
	 link[i].y = dummy_y;
        }
		
		/*read the control points of the robot*/
	tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
        tch = getc(fp);

        ungetc(tch, fp);
        fscanf(fp, " %d", num_cntl_points);
	for(i=0; i<*num_cntl_points; i++) 
	 fscanf(fp, " %d", &(cntl_points[i]));

		/*read the initial and the final configuration*/
        tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
        tch = getc(fp);
	for(i=0; i<DOF; i++) { 
	 fscanf(fp, " %d", &(begin_pos[i]));
	}
	printf("Reading initial conf "); print_conf(begin_pos);
	tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
        tch = getc(fp);
	for(i=0; i<DOF; i++) { 
         fscanf(fp, " %d", &(end_pos[i]));
/*
	 printf("r: %d\n", end_pos[i]);
*/
	}
	printf("Reading final conf "); print_conf(end_pos);

		/*read the base point of the robot*/
	tch = getc(fp);
        tch = getc(fp);
        while (tch != '\n') tch = getc(fp);
	fscanf(fp, " %d %d", basex, basey);
	ON_COMMENTS(printf("Input file %s read.\n", fname);)

}



/**************************************************************************
	read_path  reads a sequense of configurations from file fname
***************************************************************************/

void read_path(fname, path_length, path)
char *fname;
int *path_length;
configuration path[MAXPATH];
{
        FILE    *fp;
        int     dof, l;

	ON_COMMENTS(printf("Reading path from file %s.\n", fname);)
        if(!(fp = fopen(fname,"r"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }
	
	fread(&l, sizeof(int), 1, fp);
		/*read the dof for error checking purposes*/
	fread(&dof, sizeof(int), 1, fp);
	if (dof!=DOF) {
	 printf("You have to specify the graphics for this robot.\n");
	 printf("Graphics not available for a %d DOF robot.\n", dof); 
	 exit(-1);
	}
	*path_length = l;
	printf("Path length read %d\n", *path_length);
	fread(path, sizeof(configuration), l, fp);

	fclose(fp);
	ON_COMMENTS(printf("Path read from file %s.\n", fname);)
	
} 

/**************************************************************************
		procedures for writting and reading all
		data files used by the planner
***************************************************************************/
void write_path(fname, path_length, path, dof)
char *fname;
int path_length;
configuration path[MAXPATH];
int dof;
{
	int	i, t, st, rest;
        FILE    *fp;

	ON_COMMENTS(printf("Writing the path in the file %s.\n", fname);)
        if(!(fp = fopen(fname,"w"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }
	
	fwrite(&path_length, sizeof(int), 1, fp);
		/*DOF written there for checking purposes*/
	fwrite(&dof, sizeof(int), 1, fp);
	fwrite(path, sizeof(configuration), path_length , fp);

	fclose(fp);
	ON_COMMENTS(printf("Path written in file %s.\n", fname);)
} 
	 

void write_cspace(fname, cspace, dim, thetadim)
char *fname;
unsigned int cspace[DIM][DIM][DIM/32];
int dim, thetadim;
{
	FILE *fp;

	ON_COMMENTS(printf("Writing the cspace in the file %s.\n", fname);)
        if(!(fp = fopen(fname, "w"))){
         fprintf(stderr,"Cannot open file %s.\n", fname);
         exit(-1);
        }
	
	fwrite(&dim, sizeof(int), 1, fp);
	fwrite(&dim, sizeof(int), 1, fp);
	fwrite(&thetadim, sizeof(int), 1, fp);
	fwrite(cspace, sizeof(unsigned int), DIM*DIM*DIM/32, fp);
	fclose(fp);
	ON_COMMENTS(printf("Cspace written to file %s.\n", fname);)
}


void read_cspace(fname, cspace, dim, thetadim)
char *fname;
unsigned int cspace[DIM][DIM][DIM/32];
int *dim, *thetadim;
{
	FILE *fp;

	ON_COMMENTS(printf("Reading the cspace in the file %s.\n", fname);)
        if(!(fp = fopen(fname, "r"))){
         fprintf(stderr,"Cannot open file %s.\n", fname);
         exit(-1);
        }
	fread(dim, sizeof(int), 1, fp);
	fread(dim, sizeof(int), 1, fp);
	fread(thetadim, sizeof(int), 1, fp);
	printf("Reads done\n");
	if(*dim!=DIM) {
	 printf("Dim read from cpsace file not correct (%d) \n", *dim);
	 exit(-1);
	}
	fread(cspace, sizeof(unsigned int), DIM*DIM*DIM/32, fp);
	fclose(fp);
	ON_COMMENTS(printf("Cspace read from file %s.\n", fname);)
}


void write_distance(fname, dist, dim)
char *fname;
int dist[DIM][DIM];
int dim;
{
	FILE 	*fp;

	ON_COMMENTS(printf("Writing the distance in the file %s.\n", fname);)
        if(!(fp = fopen(fname, "w"))){
         fprintf(stderr,"Cannot open file %s.\n", fname);
         exit(-1);
        }
	
	fwrite(&dim, sizeof(int), 1, fp);
	fwrite(&dim, sizeof(int), 1, fp);
	fwrite(dist, sizeof(int), DIM*DIM, fp);
	fclose(fp);
	ON_COMMENTS(printf("Distance written to file %s.\n", fname);)
}


void read_distance(fname, dist, dim)
char *fname;
int dist[DIM][DIM];
int *dim;
{
	FILE 	*fp;

	ON_COMMENTS(printf("Reading the distance from the file %s.\n", fname);)
        if(!(fp = fopen(fname, "r"))){
         fprintf(stderr,"Cannot open file %s.\n", fname);
         exit(-1);
        }
	
	fread(dim, sizeof(int), 1, fp);
	fread(dim, sizeof(int), 1, fp);
	fread(dist, sizeof(int), DIM*DIM, fp);
	fclose(fp);
	ON_COMMENTS(printf("Distance read from file %s .\n", fname);)
}


/***************************************************************************
	for writing and reading the connected components 
	created by straight lines 
****************************************************************************/ 

void write_graph(fname, no_inter, graph, dof, components, component_array)
char *fname;
int no_inter;
unsigned int graph[MAX_INTER][MAX_INTER/32];
int dof, components;
int component_array[MAX_INTER];
{
        FILE    *fp;

        ON_COMMENTS(printf("Writing the graph in the file %s\n", fname);)
        if(!(fp = fopen(fname,"w"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }

        fwrite(&no_inter, sizeof(int), 1, fp);
                /*DOF written there for checking purposes*/
        fwrite(&dof, sizeof(int), 1, fp);
        fwrite(graph, sizeof(unsigned int), no_inter*(MAX_INTER/32), fp);
	fwrite(&components, sizeof(int), 1, fp);
	fwrite(component_array, sizeof(int), no_inter, fp);
        fclose(fp);
        ON_COMMENTS(printf("Graph written in file %s.\n", fname);)
}


void read_graph(fname, no_inter, graph, components, component_array)
char *fname;
int *no_inter;
unsigned int graph[MAX_INTER][MAX_INTER/32];
int *components;
int component_array[MAX_INTER];
{
        FILE    *fp;
        int     dof, l;

        ON_COMMENTS(printf("Reading graph from file %s .\n", fname);)
        if(!(fp = fopen(fname,"r"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }
        fread(&l, sizeof(int), 1, fp);
                /*read the dof for error checking purposes*/
        fread(&dof, sizeof(int), 1, fp);
        if (dof!=DOF) {
	 printf("Not prepared to handle a %d dof robot.\n", dof);
         exit(-1);
        }
        *no_inter= l;
        printf("Intermediate points read %d\n", l);
        fread(graph, sizeof(unsigned int), l*(MAX_INTER/32), fp);
	fread(&l, sizeof(int), 1, fp);
	*components = l;
	fread(component_array, sizeof(int), (*no_inter), fp);

        fclose(fp);
        ON_COMMENTS(printf("Graph read from file %s .\n", fname);)

}



/*************************************************************************/
void write_path_and_estimate(fname, path_length, path, estimate,
				 dof, avgc, minc, maxc)
char *fname;
int path_length;
configuration path[MAX_INTER];
int estimate[MAX_INTER];
int dof;
double 	avgc, minc, maxc;
{
        int     i, t, st, rest;
        FILE    *fp;

        ON_COMMENTS(printf("Writing the path and estimates in the file %s.\n", 
				fname);)
        if(!(fp = fopen(fname,"w"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }
        fwrite(&path_length, sizeof(int), 1, fp);
	printf("path_len =%d\n", path_length);
                /*DOF written there for checking purposes*/
        fwrite(&dof, sizeof(int), 1, fp);
        fwrite(path, sizeof(configuration), path_length , fp);
	fwrite(estimate, sizeof(int), path_length , fp);
	fwrite(&avgc, sizeof(double), 1, fp);
	fwrite(&minc, sizeof(double), 1, fp);
	fwrite(&maxc, sizeof(double), 1, fp);

        fclose(fp);
        ON_COMMENTS(printf("Path written in file %s.\n", fname);)
}

void read_path_and_estimate(fname, path_length, path, estimate,
                                 avgc, minc, maxc)
char *fname;
int *path_length;
configuration path[MAX_INTER];
int estimate[MAX_INTER];
double  *avgc, *minc, *maxc;
{
        int     i, t, st, rest, l, dof, pl;
        FILE    *fp;

        ON_COMMENTS(printf("Reading the path and estimates from file %s.\n", 
                                fname);)
        if(!(fp = fopen(fname,"r"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }
        fread(&pl, sizeof(int), 1, fp);
	printf("length read %d\n", pl);
                /*DOF written there for checking purposes*/
        fread(&dof, sizeof(int), 1, fp);
	if (dof!=DOF) {
         printf("You have to specify the graphics for this robot.\n");
         printf("Graphics not available for a %d DOF robot.\n", dof);
         exit(-1);
        }
	*path_length = pl;
        fread(path, sizeof(configuration), pl, fp);
        fread(estimate, sizeof(int), pl, fp);
        fread(avgc, sizeof(double), 1, fp);
        fread(minc, sizeof(double), 1, fp);
        fread(maxc, sizeof(double), 1, fp);

        fclose(fp);
        ON_COMMENTS(printf("Path read from file %s.\n", fname);)
}


/***********************************************************************
        Concatenate path1 to path2
***********************************************************************/

void concat(path1, path2, len1, len2)
configuration path1[MAXPATH/2],path2[MAXPATH];
int  len1, *len2;
{
        int i, start_dim;

        start_dim = *len2;
        *len2 = len1 + *len2;
        if(*len2>MAXPATH) {
         printf("Path found exceeds toatal length of path that can be kept.\n");         exit(-1);
        }
        for(i=0; i<len1 ; i++) {
         assign_conf(path1[i], path2[i + start_dim]);
        }
}



