/*****************************************************************************

  COPYRIGHT (c) 1995
  THE BOARD OF TRUSTEES OF THE LELAND STANFORD JUNIOR UNIVERSITY
  ALL RIGHTS RESERVED

  No duplications, whole or partial, manual or electronic, may be made
  without the express permission of the author. Any such copies, or
  revisions thereof, must display this notice unaltered.

  Robotics Laboratory,
  Computer Science Department,
  Stanford University

  Original author:
  Lydia Kavraki (kavraki@cs.stanford.edu)

******************************************************************************/


#include "defs.h"

int		ttmp_path_len;
configuration	ttmp_path[MAXPATH/10];

void init_store_str(graph, no_inter)
cellp   *graph[MAX_INTER];
int     no_inter;
{
        int i;

        for(i=0; i<no_inter; i++) 
         graph[i] = NULL;
}

/***********************************************************************
	stores the infor the i, j are connected. 
	If the path has to be stored (path is produced by an expensive
		path planner like random bounce, rpp), 
	        it is stored only ONCE in the (i,j) cell with i<j.
***********************************************************************/

		/*store both connections i, j and j , i*/
void set_store_str(graph, i, j, val, path_type, path_length, path) 
cellp 	*graph[MAX_INTER];
int 	i, j, val;
int	path_type, path_length;
configuration *path;
{
	cellp 	*new_cell;
	int 	k, l, min_val, max_val;
	
	if(val == 0) return;
	if(i == j) return;

		/*store the path ONCE*/
	min_val = my_MIN(i, j);
	max_val = my_MAX(i, j);

		/*create the cell to store*/
	new_cell = get(cellp, 1);
        new_cell->final = max_val;
        new_cell->path_type = path_type;
        new_cell->path_length = path_length;
	if(path_length==0) 
	 new_cell->path = NULL;
	else {		/*copy the path*/
	 new_cell->path = get(configuration, path_length); 
	 if(min_val == i) 
          memcpy(new_cell->path, path, DOF*path_length*sizeof(int));
	 else {
	  for(k=path_length - 1, l=0; k>=0; k--, l++)
	   assign_conf(path[k], new_cell->path[l]);
         }
	} /*else*/

			/*first the case there is no element.*/
	if(graph[min_val] == NULL) {
	 graph[min_val] = new_cell;
	 new_cell->next = NULL;
	}
	else {
	 new_cell->next = graph[min_val];
	 graph[min_val] = new_cell;
	}

			/*store the other connection only but not the 
			  entire path*/
	new_cell = get(cellp, 1);
        new_cell->final = min_val;
	new_cell->path_type = path_type;
        new_cell->path_length = -path_length;
	new_cell->path = NULL;
	if(graph[max_val] == NULL) {
         graph[max_val] = new_cell;
         new_cell->next = NULL;
        }
        else {
         new_cell->next = graph[max_val];
         graph[max_val]  = new_cell;
        }
}





		/*this reads only the value and the type of the path*/
int read_store_str(graph, i, j, path_type)     
cellp 	*graph[MAX_INTER];
int 	i, j, *path_type;
{
	cellp  *tmp;

	if(i==j) return 1;
	tmp = graph[i];
	if(!tmp) {*path_type = 0; return 0;}
	while((tmp->final != j) && (tmp->next)) tmp = tmp->next;
        if(tmp->final == j) {
	 *path_type = tmp->path_type;
	 return 1; 
	} 
	else {
	 *path_type = 0;
         return 0;
	}
}


		/*this reads the path of a connection that exists*/
int get_path(graph, i, j,  path_length, path)
cellp 		*graph[MAX_INTER];
int		i, j;
int		*path_length;
configuration 	*path;
{
	cellp  		*tmp;
        int     	min_val, max_val, len, k, l, flag;

        min_val = my_MIN(i, j);
        max_val = my_MAX(i, j);
        tmp = graph[min_val];
	if((j==24) && (i==633))
	 printf("in get path: try to connect\n");

                /*check if empty*/
        if (tmp==NULL) return 0;
        while((tmp->final != max_val) && (tmp->next))
         tmp = tmp->next;
	if(tmp->final != max_val) return 0;

	if((j==24) && (i==633))
	 printf("in get path: type %d length %d i %d min_val %d\n", 
			tmp->path_type, tmp->path_length, i, min_val);
		/*connection is found*/
	switch(tmp->path_type) {
			case CONNECT_RPP_LINE:
	 /*path is strored get the right path*/
	 if(i==min_val) {  
	   *path_length = len = tmp->path_length;
	   memcpy(path, tmp->path, len*sizeof(configuration));
	 }
	 else { 
		/*the reverse connection must exist*/
	  for(tmp=graph[min_val]; tmp && (tmp->final!=i); tmp=tmp->next); 
	  *path_length = len = my_ABS(tmp->path_length);
	  for(k=len-1, l=0; k>=0; k--, l++)
           assign_conf(tmp->path[k], path[l]);
	 }
	 break;

			case CONNECT_LINE: 
	 /*produce the path now*/
	 flag = connect_simple(Inter_p[i],Inter_p[j],1,path_length,path);
	 if(!flag) {
	  flag = connect_simple(Inter_p[j], Inter_p[i], 1, 
					&ttmp_path_len, ttmp_path);
	  if(!flag){
	   printf("Serious: problem in both directions of connection.\n");
	   printf("Conf %d and conf %d\n", j, i);
	   print_conf(Inter_p[j]);
	   print_conf(Inter_p[i]);
	   exit(-1);
	  }
	  for(k=ttmp_path_len-1, l=0; k>=0; k--, l++)
	   assign_conf(ttmp_path[k], path[l]);
	  *path_length = ttmp_path_len;
	 }
	 break;
			default:
	 printf("Invalid case in get_path\n");
	 exit(-1);
	} /*switch*/
	
	return 1;
}



/**********************************************************************
		find the connected components
**********************************************************************/

void dfs(graph, node, component, no_inter, visited)
cellp *graph[MAX_INTER];
int node, component, no_inter;
int visited[MAX_INTER];
{
        int 	j, path_type;
	cellp	*tmp;

        visited[node] = component;
	tmp = graph[node];
	while(tmp) {
	 if(visited[tmp->final] == -1) 
          dfs(graph, tmp->final, component, no_inter, visited);
	 tmp = tmp->next;
        }
}



int connected_components(graph, no_inter, component_array)
cellp 	*graph[MAX_INTER];
int 	no_inter;
int 	component_array[MAX_INTER];
{
	int i, count, connected_comp;

	for(i=0; i<no_inter; i++) 
	 component_array[i] = -1;

	count = connected_comp = 0;
	while(count<no_inter) {
	 dfs(graph, count, connected_comp, no_inter, component_array);
	 connected_comp++;
	 while((count<no_inter) && (component_array[count] != -1)) count++;
	}
	return connected_comp;
}




/*************************************************************************
	write to a file everything the planner needs
**************************************************************************/
void write_graph_and_paths(fname, no_inter, inter_p, 
		 	component_number, in_component,  graph)
char 		*fname;
int 		no_inter, component_number;
configuration	inter_p[MAX_INTER];
int		in_component[MAX_INTER];
cellp		*graph[MAX_INTER];
{
	FILE 		*fp;
        int     	i, j, count, dof = DOF;
	cellp 		*cell;


        ON_COMMENTS(printf("Writing the graph and paths in the file %s\n",
						fname);)
        if(!(fp = fopen(fname,"w"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }

        fwrite(&no_inter, sizeof(int), 1, fp);
        fwrite(&dof, sizeof(int), 1, fp);
	fwrite(inter_p, sizeof(configuration), no_inter, fp);
	fwrite(&component_number, sizeof(int), 1, fp);
	fwrite(in_component, sizeof(int), no_inter, fp);

	printf("In write_graph_and_paths, initial done\n");

	for(i=0; i<no_inter; i++) {
	 cell = graph[i]; 
	 count = 0;
	 while(cell) {
	  cell = cell->next; count++;
	 }
	 fwrite(&i, sizeof(int), 1, fp);
	 fwrite(&count, sizeof(int), 1, fp);
	 cell = graph[i];
	 for(j=0; j<count; j++) { 
	  fwrite(&(cell->final), sizeof(int), 1, fp);
	  fwrite(&(cell->path_type), sizeof(int), 1, fp);
	  fwrite(&(cell->path_length), sizeof(int), 1, fp);
	  if (cell->path_length>0) 
	   fwrite(cell->path, sizeof(configuration), cell->path_length, fp);
	  cell = cell->next;
	 }  
	}
        fclose(fp);
        ON_COMMENTS(printf("Graph written in file %s.\n", fname);)
}




void read_graph_and_paths(fname, no_inter, inter_p, component_number, 
				in_component, graph)
char 		*fname;
int 		*no_inter, *component_number;
configuration   inter_p[MAX_INTER];
int             in_component[MAX_INTER];
cellp 		*graph[MAX_INTER];
{
        FILE    	*fp;
        int     	i, j, num, count, tmp;
	cellp 		*cell;

        ON_COMMENTS(printf("Reading the graph and paths from file %s\n",
						fname);)
        if(!(fp = fopen(fname,"r"))){
         fprintf(stderr,"Cannot open file %s\n",fname);
         exit(-1);
        }

        fread(&num, sizeof(int), 1, fp);
	*no_inter = num;
        fread(&tmp, sizeof(int), 1, fp);
	if(tmp!=DOF) {
	 printf("Wrong DOF.\n"); exit(-1);
	}
	fread(inter_p, sizeof(configuration), num, fp);
	fread(&tmp, sizeof(int), 1, fp);
	*component_number = tmp;
	fread(in_component, sizeof(int), num,  fp);


	
	init_store_str(graph, num);
	for(i=0; i<num; i++) {
	 fread(&count, sizeof(int), 1, fp);
	 if(i!=count) {
	  printf("Error when reading connections %d %d.\n", i, count);
	  exit(-1);
	 }
	 fread(&count, sizeof(int), 1, fp);
	 for(j=0; j<count; j++) { 
	  cell = get(cellp, 1);
	  fread(&(cell->final), sizeof(int), 1, fp);
	  fread(&(cell->path_type), sizeof(int), 1, fp);
	  fread(&(cell->path_length), sizeof(int), 1, fp);
	  if (cell->path_length>0) { 
	  cell->path = get(configuration, cell->path_length);
	  fread(cell->path, sizeof(configuration),
		cell->path_length, fp);
	  } else cell->path = NULL;
	  cell->next = graph[i];
	  graph[i] = cell;
	 }  
	} /*i*/
        fclose(fp);
        ON_COMMENTS(printf("Graph and paths read in file %s.\n", fname);)
}

